/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "stdafx.h"
#include "pim/maincpp.h"
#include "pim/SettingFunctions.h"
#include "base/startcmd.h"
#include "notify/timed_msgbox.h"
#include "HwndFunctions.h"
#include "spds/constants.h"
#include "Winuser.h"

#include "ui.h"
#include "ClientUtil.h"
#include "base/util/utils.h"
#include "processUtils.h"
#include "pim/ClientSettings.h"
#include "pim/utils.h"



// items in listboxes MUST be load in this order
int SynctypeReg2UI(const char* sSyncType) {
    int pos = 0;

    if(!strcmp(sSyncType, SYNC_MODE_TWO_WAY)) {
        pos = 0;
    } else if(!strcmp(sSyncType, SYNC_MODE_ONE_WAY_FROM_SERVER) ||
              !strcmp(sSyncType, "one-way-server")) {   // old value, keep for compatibility!
        pos = 1;
    } else if(!strcmp(sSyncType, SYNC_MODE_ONE_WAY_FROM_CLIENT) ||
              !strcmp(sSyncType, "one-way-client")) {   // old value, keep for compatibility!
        pos = 2;
    }
    return pos;
}


const char* SynctypeUI2Reg(int nPos) {
    switch(nPos) {
        case 0:
            return SYNC_MODE_TWO_WAY;             break;
        case 1:
            return SYNC_MODE_ONE_WAY_FROM_SERVER; break;
        case 2:
            return SYNC_MODE_ONE_WAY_FROM_CLIENT; break;
    }
    // should not get here
    return "none";
}

const char* getSyncModeAtPos(CExtItem* item, const int pos) {

    if (!item) return NULL;
    WindowsSyncSourceConfig* wssc = item->getSourceConfig();
    if (!wssc) return NULL;

    ArrayList modes;
    StringBuffer s = wssc->getSyncModes();
    s.split(modes, ",");

    int count=0;
    for (int i=0; i<modes.size(); i++) {
        StringBuffer* e = (StringBuffer*)modes.get(i);
        if (e) {
            if (*e == SYNC_MODE_TWO_WAY ||                  // these are the syncmodes
                *e == SYNC_MODE_ONE_WAY_FROM_CLIENT ||      // available from the UI
                *e == SYNC_MODE_ONE_WAY_FROM_SERVER ||
                *e == SYNC_MODE_SMART_ONE_WAY_FROM_SERVER ||
                *e == SYNC_MODE_SMART_ONE_WAY_FROM_CLIENT) {
                if (pos == count) {
                    return e->c_str();
                }
                count ++;
            }
        }
    }

    // not found (!) return the default
    if (isPIMSource(item->getId())) {
        return DEFAULT_PIM_SYNC_MODE;
    } 
    else if (item->getName() == SOURCE_NAME_PICTURE) {
        return DEFAULT_PICTURES_SYNC_MODE;
    }
    else {
        return DEFAULT_MAIL_SYNC_MODE;
    }
}


/**
 * @return -1= NO, 0:YES, 1=Not running
 */
int EndAppQuery(HWND wndUIWindow) {

    int ret = -1;
    CString s1;
    int processID = 0;
    HWND wnd = NULL;
    wnd = ::FindWindow(TEXT("funambolstartsync"), NULL);
    if ((processID = checkStartSync()) != 0)
    {
        // sync is in progress
        HwndFunctions::closePreviousMsgBox();
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_INTERRUPT_SYNC);
        //s1 = "Do you really want to interrupt the synchronization?";
        int retMsgBox = TimedMessageBox(wndUIWindow, s1, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
            MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, 10 * 1000);

        if(retMsgBox == IDYES)
        {
            // send msg to startsync to stop
            if(wnd){
                ::PostMessage(wnd, WM_COMMAND, (WPARAM)10, NULL);
            }
            Sleep(500);
            HANDLE h = OpenProcess(0, FALSE, processID);
            if (h) {
                DWORD code = 0;
                GetExitCodeProcess(h, &code);
                if (code == STILL_ACTIVE) {
                    startProgram(SYNCAPP, TEXT("removeNotif"));
                    TerminateProcess(h, -6); //-6 code for the user stop sync.
                    CloseHandle(h);
                    setClientConfiguration(NULL, PROPERTY_SPDM_PID, TEXT("0"), NULL);
                    CString s; s.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CODE_SYNC_STOPPED);
                    LOG.info(_wcc(s.GetString()));
                }
            }
         
            ClientSettings *config = getRegConfig();
            MailSyncSourceConfig* mssc = config->getMailSyncSourceConfig(true);
            MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, "mails");
            if (notifyMailAccountsCreated(wndUIWindow)) {
                if (mssc) {
                    int accountsNumber = mssc->getMailAccounts().size();

                    for (int i=0; i < accountsNumber; i++) {
                        MailAccount* account = (MailAccount*)mssc->getMailAccounts().get(i);
                        if (account) {
                            if (account->getDirty()) {
                                account->setDirty(false);
                            }
                        }
                    }
                    m.setMailSourceConfig(*mssc);
                }
            }
            // The user answered YES
            return 0;
        }
    }
    else {
        // send msg to startsync to remove the notif Icon if there is again
        startProgram(SYNCAPP, TEXT("removeNotif"));
        // The process is not running
        return 1;
    }
    // The user answered no
    return -1;
}


int getSyncModeCode(const char* syncMode){
    int code = SYNC_NONE;

    if(strcmp(syncMode, "none") == 0)
        code = SYNC_NONE;
    if(strcmp(syncMode, "two-way") == 0)
        code = SYNC_TWO_WAY;
    if(strcmp(syncMode, "slow") == 0)
        code = SYNC_SLOW;
    if(strcmp(syncMode, "one-way-from-client") == 0)
        code = SYNC_ONE_WAY_FROM_CLIENT;
    if(strcmp(syncMode, "refresh-from-client") == 0)
        code = SYNC_REFRESH_FROM_CLIENT;
    if(strcmp(syncMode, "one-way-from-server") == 0)
        code = SYNC_ONE_WAY_FROM_SERVER;
    if(strcmp(syncMode, "refresh-from-server") == 0)
        code = SYNC_REFRESH_FROM_SERVER;

    return code;
}

const char* getSyncModeName(int code) {
    switch (code) {
        case SYNC_NONE:
            return "none";
            break;
        case SYNC_TWO_WAY:
        case SYNC_TWO_WAY_BY_SERVER:
            return "two-way";
            break;
        case SYNC_SLOW:
            return "slow";
            break;
        case SYNC_ONE_WAY_FROM_CLIENT:
        case SYNC_ONE_WAY_FROM_CLIENT_BY_SERVER:
            return "one-way-from-client";
            break;
        case SYNC_REFRESH_FROM_CLIENT:
        case SYNC_REFRESH_FROM_CLIENT_BY_SERVER:
            return "refresh-from-client";
            break;
        case SYNC_ONE_WAY_FROM_SERVER:
        case SYNC_ONE_WAY_FROM_SERVER_BY_SERVER:
            return "one-way-from-server";
            break;
        case SYNC_REFRESH_FROM_SERVER:
        case SYNC_REFRESH_FROM_SERVER_BY_SERVER:
            return "refresh-from-server";
            break;
        default:
            return NULL;
            break;
    }
}

BOOL existsDirectory(wchar_t* dir) {

    WIN32_FIND_DATA FileData;
    BOOL ret = TRUE;
    HANDLE hFind;

    hFind = FindFirstFile(dir, &FileData);

    if (hFind == INVALID_HANDLE_VALUE) {
        ret = FALSE;
    } else {
        ret = TRUE;
    }
    if (hFind) {
        FindClose(hFind);
    }

    return ret;
}

/*
 * Modify the window position and size of the specified offset.
 * 
 * See ClientUtil.h for the doc.
 */
void modifyWindow(CWnd *wnd, int x, int y, int cx, int cy) {
    RECT rc;
    wnd->GetWindowRect(&rc);
    rc.left   += x;
    rc.top    += y;
    rc.right  += x + cx;
    rc.bottom += y + cy;
    
    wnd->MoveWindow(&rc);
}

/*
 * Modify the window size to fit the parent one.
 * 
 * See ClientUtil.h for the doc.
 */
void adjustToParent(CWnd *wnd, int opt, int lx, int rx, int ty, int by) {
    RECT rc, parentRect;

    wnd->GetWindowRect(&rc);
    wnd->GetParent()->GetWindowRect(&parentRect);

    if (opt & ADJUST_WIDTH) {
        rc.left   = parentRect.left + lx;
        rc.right  = parentRect.right - rx;
    }
    if (opt & ADJUST_HEIGHT) {
        rc.top    = parentRect.top + ty;
        rc.bottom = parentRect.bottom - by;
    }
    wnd->MoveWindow(&rc);
}

CString escapeAmpString(CString s) {
    
    CString current = s;
    int pos = 0;
    
    while(pos != -1 && pos <= current.GetLength()) {
        pos = current.Find('&', pos);
        if (pos != -1) {
            current.Insert(pos, '&');
            pos += 2;
        }
    }
    return current;
}

void drawLine(CDC *dc, CPoint& start, CPoint& end, COLORREF color, int width, int penStyle) {

    // Set the pen and save tho old one
	CPen pen(penStyle, width, color);
    CPen* oldPen = dc->SelectObject(&pen);

	dc->MoveTo(start.x, start.y);
	dc->LineTo(end.x,   end.y);

    // Restore the old pen
	dc->SelectObject(oldPen);
}


void EnableButton(HWND dialog, int buttonID, bool bEnable) {

    HWND hwndMB = SHFindMenuBar(dialog);
    TBBUTTONINFO tbi;
    if (!hwndMB)
        return;

    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    SendMessage(hwndMB, TB_GETBUTTONINFO, buttonID, (LPARAM)&tbi);

    if(bEnable)  tbi.fsState = TBSTATE_ENABLED;
    else         tbi.fsState &= ~TBSTATE_ENABLED;

    SendMessage(hwndMB, TB_SETBUTTONINFO, buttonID, (LPARAM)&tbi);
}




DWORD WINAPI configWorker(LPVOID lpv)
{
    LOG.debug("Starting configWorker thread");

    // The dialog HWND that called this function.
    HWND dialogWnd = (HWND)lpv;
    CString s1;

    //
    // Start the config sync, in a process "startsync.exe config".
    //
    PROCESS_INFORMATION procinfo;
    DWORD exitcode = -2;
    wchar_t *path = NULL;
    wstring cmd;
    ClientSettings* cs = getRegConfig();
    path = toWideChar(cs->getAppPath().c_str());
    
    if (path) {
        cmd += path;
        delete [] path;
    }
    cmd += TEXT("\\startsync.exe");

    //
    // SYNC TYPE: keep 'refresh-from-server' if it's set from registry.
    //            Otherwise it's a two-way.
    wstring configSyncType;
    SyncSourceConfig* ssc = cs->getSyncSourceConfig("config");
    if (!ssc) {
        LOG.error("Config Syncsource not existing, aborting config sync.");
        goto finally;
    }
    if (!strcmp(ssc->getSync(), "refresh-from-server")) { 
        configSyncType = TEXT("refresh-config"); 
    }
    else { 
        // default
        configSyncType = TEXT("two-way-config"); 
    }

    configSyncType += TEXT(" config");


    //-----------------------------------------------------------
    if (!CreateProcess(cmd.c_str(), configSyncType.c_str(),
                       NULL, NULL, FALSE, 0,
                       NULL, NULL, NULL, &procinfo ) ) {
        LOG.error("%s process not created: error", cmd.c_str());
        goto finally;
    }
    //-----------------------------------------------------------
    
    int timeout = 120;
    DWORD waitResult = WaitForSingleObject(procinfo.hProcess, timeout*1000);
    switch (waitResult) {
        // Thread exited, get exit code.        
        case WAIT_ABANDONED:
            LOG.debug("refresh-config abandoned");
            exitcode = -1;
            break;

        case WAIT_OBJECT_0: {
            ::SetForegroundWindow(dialogWnd);
            GetExitCodeProcess(procinfo.hProcess, &exitcode);
            LOG.debug("configSync ended with code %d", exitcode);            
            break;
        }
        
        // Thread is still running after timeout -> kill it and exit.
        case WAIT_TIMEOUT: {
            LOG.debug("Timeout - configSync will now be terminated");
            TerminateProcess(procinfo.hProcess, 1);            
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NETWORK_ERROR);
            AfxMessageBox(s1);
            exitcode = -3;
            break;
        }

        // Some error occurred (case WAIT_FAILED)
        default: {
            LOG.debug("Wait error on configSync");
            TerminateProcess(procinfo.hProcess, 1);
            exitcode = -4;
            break;
        }                     
    }

finally:
    // Close the dialog
    // SendMessage(dialogWnd, ID_MYMSG_CONFIG_SYNC_ENDED, exitcode, NULL);
    SendNotifyMessage(dialogWnd, ID_MYMSG_CONFIG_SYNC_ENDED, exitcode, NULL);
    return 0;
}


bool gotoAccount(CString accountName) {
    
    wstring app = TEXT("tmail.exe");        
    wstring arg = TEXT("-service \"");
    arg.append(accountName) ;
    arg.append(TEXT("\" -open \"inbox\""));
    
    PROCESS_INFORMATION procinfo; 
    return CreateProcess( app.c_str(), arg.c_str(),
                       NULL, NULL, false, 0,
                       NULL, NULL, NULL, &procinfo ) == TRUE ? true : false;

}

bool gotoApplication(int id) {
    
    wstring app;
    wstring arg;

    PROCESS_INFORMATION procinfo; 
    if (id == SOURCE_MAIL) {
        app = TEXT("tmail.exe");        
        arg = TEXT("-service \"");
        arg.append(DEFAULT_ACCOUNT_NAME) ;
        arg.append(TEXT("\" -open \"inbox\""));
    } else if (id == SOURCE_CONTACTS) {
        app = TEXT("poutlook.exe");
        arg.append(TEXT("contacts -vd"));
    } else if (id == SOURCE_CALENDAR) {
        app = TEXT("poutlook.exe");
        arg.append(TEXT("calendar -vd"));
    } else if (id == SOURCE_TASKS) {
        app = TEXT("poutlook.exe");
        arg.append(TEXT("tasks -vd"));
    } else if (id == SOURCE_NOTES) {
        app = TEXT("notes.exe");
        arg = TEXT("");
    } else if (id == SOURCE_BRIEFCASE) {
        app = TEXT("fexplore.exe");

        WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig("briefcase");
        string d = ssconf->getSyncDir();
        wchar_t* t = toWideChar(d.c_str());
        if (existsDirectory(t)) {
            arg = t;
        } else {
            wchar_t pathRetrieved[512];
            SHGetSpecialFolderPath(NULL, pathRetrieved, CSIDL_PERSONAL, 0);   
            arg = pathRetrieved;
        }
        delete [] t;
    } else if (id == SOURCE_PICTURES) {        
        app = TEXT("pimg.exe");
        WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig(SOURCE_NAME_PICTURE);
        string d = ssconf->getSyncDir();
        wchar_t* t = toWideChar(d.c_str());
        if (existsDirectory(t)) {
            arg = TEXT("-directory:");
            arg += t;
        } else {
            wchar_t pathRetrieved[512];
            SHGetSpecialFolderPath(NULL, pathRetrieved, CSIDL_MYPICTURES, 0);   
            arg = pathRetrieved;
        }
        delete [] t;
    } 

    return CreateProcess( app.c_str(), arg.c_str(),
                       NULL, NULL, false, 0,
                       NULL, NULL, NULL, &procinfo ) == TRUE ? true : false;
    
}


static void trim(wstring& str) {
    wstring::size_type pos = str.find_last_not_of(' ');
    if(pos != string::npos) {
        str.erase(pos + 1);
        pos = str.find_first_not_of(' ');
        if(pos != string::npos) 
            str.erase(0, pos);
    }
    else 
        str.erase(str.begin(), str.end());
}

wstring reformatDate(wstring date) {
    
    int found = 0;
    // remove "dddd" or "dddd," or "dddd, "
    if ((found = date.find(TEXT("dddd, "))) != wstring::npos) {
        date.replace(found, 6, TEXT(""));
    } else if ((found = date.find(TEXT("dddd,"))) != wstring::npos) {
        date.replace(found, 5, TEXT(""));
    }else if ((found = date.find(TEXT("dddd"))) != wstring::npos) {
        date.replace(found, 4, TEXT(""));
    }
    
    trim(date);
    // change full month in abbreviation (e.g. September into Sept)    
    if ((found = date.find(TEXT("MMMM, "))) != wstring::npos) {
        date.replace(found, 6, TEXT("MMM, "));
    } else if ((found = date.find(TEXT("MMMM,"))) != wstring::npos) {
        date.replace(found, 5, TEXT("MMM,"));
    } if ((found = date.find(TEXT("MMMM"))) != wstring::npos) {
        date.replace(found, 4, TEXT("MMM"));
    }
    
    trim(date);
    // remove year
    if (((found = date.find(TEXT("yyyy, "))) != wstring::npos) || 
        ((found = date.find(TEXT(", yyyy"))) != wstring::npos)) {
        date.replace(found, 6, TEXT(""));
    } else if (((found = date.find(TEXT("yyyy,"))) != wstring::npos) ||
               ((found = date.find(TEXT(",yyyy"))) != wstring::npos)) {
        date.replace(found, 5, TEXT(""));
    } if ((found = date.find(TEXT("yyyy"))) != wstring::npos) {
        date.replace(found, 4, TEXT(""));
    }
    trim(date);
    
    return date;

}

wstring reformatTime(wstring time) {
    
    int found = 0;
    // remove ":ss" or ":ss," or "ss"
    if ((found = time.find(TEXT(":ss,"))) != wstring::npos ||
        (found = time.find(TEXT(".ss,"))) != wstring::npos) {
        time.replace(found, 4, TEXT(""));
    } else if ((found = time.find(TEXT(":ss"))) != wstring::npos ||
               (found = time.find(TEXT(".ss"))) != wstring::npos) {
        time.replace(found, 3, TEXT(""));
    }else if ((found = time.find(TEXT("ss"))) != wstring::npos) {              
        time.replace(found, 2, TEXT(""));
    }
    
    trim(time);   
    
    return time;

}

bool isToday(SYSTEMTIME time) {
    SYSTEMTIME st;
    GetLocalTime(&st);
    if (time.wYear  == st.wYear   &&
        time.wMonth == st.wMonth &&
        time.wDay   == st.wDay) {

        return true;
    }
    else {
        return false;
    }
}

wstring getFormattedDateAndTime(long lastSync) {

    SYSTEMTIME timeDest; wchar_t data[80]; 
    wchar_t formatDate[80]; wchar_t formatTime[80];
    wstring result;

    CTime timeSyncContacts(lastSync);
    
    timeSyncContacts.GetAsSystemTime(timeDest);
    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLONGDATE, data, 80);

    if (isToday(timeDest)) {
        CString s1;
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_TODAY);
        wcscpy(formatDate, s1);
    } else {
        result = reformatDate(data);   
    GetDateFormat(LOCALE_USER_DEFAULT, NULL, &timeDest, result.c_str(), formatDate, 80); 
    }
    
    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_STIMEFORMAT, data, 80);
    result = reformatTime(data);   

    GetTimeFormat(LOCALE_USER_DEFAULT, NULL, &timeDest, result.c_str(), formatTime, 80); 
    
    result = formatDate;
    result += TEXT(" ");
    result += formatTime;    
    
    return result;
}

wstring formatOnlyDateInLocale(const StringBuffer& date) {
    
    wstring dd(TEXT(""));
    wchar_t* wdate = toWideChar(date);
    if (wdate == NULL) {
        return dd;
    }
    wchar_t data[80];
    wchar_t formatDate[80];
    int found = 0;
    SYSTEMTIME timeDest;
    swscanf(wdate, L"%4d%2d%2d", &timeDest.wYear, &timeDest.wMonth, &timeDest.wDay);
    
    GetLocaleInfo(LOCALE_USER_DEFAULT, LOCALE_SLONGDATE, data, 80);
    dd = data;            
    if ((found = dd.find(TEXT("dddd, "))) != wstring::npos) {
        dd.replace(found, 6, TEXT(""));
    } else if ((found = dd.find(TEXT("dddd,"))) != wstring::npos) {
        dd.replace(found, 5, TEXT(""));
    }else if ((found = dd.find(TEXT("dddd"))) != wstring::npos) {
        dd.replace(found, 4, TEXT(""));
    }

    trim(dd);            
    GetDateFormat(LOCALE_USER_DEFAULT, NULL, &timeDest, dd.c_str(), formatDate, 80); 
    dd = formatDate;
    delete [] wdate;
    return dd;
}

bool isPIMSource(const int sourceID) {

    if (sourceID == SOURCE_CONTACTS ||
        sourceID == SOURCE_CALENDAR ||
        sourceID == SOURCE_TASKS    ||
        sourceID == SOURCE_NOTES    ||
        sourceID == SOURCE_BRIEFCASE ) {
        return true;
    }
    return false;
}

int notifyMailAccountsCreated(HWND wndUIWindow) 
{
    MailSyncSourceConfig* msc = getRegConfig()->getMailSyncSourceConfig(true);
    int accountsNumber = msc->getMailAccounts().size();   
    int accountsAddedMask = 0;
    StringBuffer names;
    int count = 0;      // the number of accounts created

    for (int i=0; i<accountsNumber; i++) {
        MailAccount* account = (MailAccount*)msc->getMailAccounts().get(i);
        if (account) {
            if (account->getDirty()) {
                LOG.info("%s: account \"%s\" has dirty flag set", __FUNCTION__,
                    account->getName().c_str());

                if (count > 0) {
                    names.append(", ");
                }
                names.append("\""); 
                names.append(account->getName()); 
                names.append("\"");

                count++;
            }
        }
    }

    //
    // Format the popup message: different if more than 1 account is created
    //
    if (count) {
        CString s1;
        const WCHAR* wnames = toWideChar(names.c_str());

        int resourceID = IDS_NEW_ACCOUNT_NOTIFICATION;
        if (count > 1) {
            resourceID = IDS_MANY_NEW_ACCOUNTS_NOTIFICATION;
        }
        s1.FormatMessage(getLocalizationUtils()->getLocalizationString(resourceID), wnames);

        LOG.info("%s: opening message box", __FUNCTION__); 
        TimedMessageBox(wndUIWindow, s1, TEXT(APP_NAME), MB_OK | MB_ICONINFORMATION, INFINITE);
    }

    return count;
}


int getPositionOfSource(CCustomList& lstSources, const int sourceID) {

    ArrayList sources;
    StringBuffer sourceOrder = getRegConfig()->getBrandingParams().getSourceOrder().c_str();
    sourceOrder.split(sources, ",");
    
    int index = 0;
    for (int i=0; i < sources.size(); i++) {
        StringBuffer* source = (StringBuffer*)sources.get(i);
        if (source) {
            int id = sourceNameToId(source->c_str());
            if (id == sourceID) {
                return index;  // found
            }
            // increment index ONLY if this source is really visible
            if (lstSources.isItemVisible(id)) {
                index ++;
            }
        }
    }
    
    // not found in sourceOrder!
    return -1;
}