/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// PIMSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "PIMSettings.h"
#include "uiDlg.h"
#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"
#include "ExploreFolders.h"
#include "pim/utils.h"

#include "pim/clientsettings.h"
#include "pim/Maincpp.h"


// CPIMSettings dialog

IMPLEMENT_DYNAMIC(CPIMSettings, CDialog)

CPIMSettings::CPIMSettings(CWnd* pParent /*=NULL*/)
             : CDialog(CPIMSettings::IDD, pParent),
               customList(((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl()) {
    
    CExtItem* cb = customList.getItem(SOURCE_NOTES);
    existsNote = cb ? true : false;

    cb = customList.getItem(SOURCE_BRIEFCASE);
    existsBriefcase = cb ? true : false;
         
    oldOrientation = -1;    
    dropDownOpened = false;
    
}

CPIMSettings::~CPIMSettings() { }


void CPIMSettings::DoDataExchange(CDataExchange* pDX) {

    CDialog::DoDataExchange(pDX);    

#if defined(WIN32_PLATFORM_WFSP)
    DDX_Control(pDX, IDC_SETTPIM_SPIN1, spinSyncType);
#endif

    DDX_Control(pDX, IDC_PIM_SYNCTYPE,  lstSyncType );
    DDX_Control(pDX, IDC_NOTES_LINK_CHOOSE,     linkNotesChoose);
    DDX_Control(pDX, IDC_BRIEFCASE_LINK_CHOOSE, linkBriefcaseChoose);
    
}

BEGIN_MESSAGE_MAP(CPIMSettings, CDialog)   

    ON_BN_CLICKED(IDC_NOTES_LINK_CHOOSE, &CPIMSettings::OnBnClickedNotesLinkChoose)
    ON_BN_CLICKED(IDC_BRIEFCASE_LINK_CHOOSE, &CPIMSettings::OnBnClickedBriefcaseLinkChoose)
    ON_WM_CTLCOLOR()
    ON_WM_SIZE()

    ON_CBN_DROPDOWN(IDC_PIM_SYNCTYPE, &CPIMSettings::OnButtonDropDown)
    ON_CBN_CLOSEUP(IDC_PIM_SYNCTYPE, &CPIMSettings::OnButtonCloseUp)
END_MESSAGE_MAP()


afx_msg void CPIMSettings::OnButtonDropDown() {
    dropDownOpened = true;
}
afx_msg void CPIMSettings::OnButtonCloseUp() {
    dropDownOpened = false;
}

bool CPIMSettings::createMenu() {   
    
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;

    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return false;
    };
    return true;    
}


void CPIMSettings::InitializeFont() { 
   
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT)GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    GetObject(hFont, sizeof pathlf, &pathlf);
    
    pathlf.lfHeight += (2 * FONT_HEIGHT_OFFSET);
    VERIFY(pathFontSize.CreateFontIndirect(&pathlf));
       
    lf.lfHeight += FONT_HEIGHT_OFFSET;
    VERIFY(defaultFontSize.CreateFontIndirect(&lf));
     
    // add the bold
    lf.lfWeight = FW_BOLD;
    VERIFY(fontBold.CreateFontIndirect(&lf)); 
    
    
}

/**
* The direction is based on the one of the first pim component
*/
void CPIMSettings::initializeSyncDirection() {
    
    int direction = -1;
    for (int i = 0; i < customList.getCountItem(); i++) {

        int id = customList.indexToId(i);
        if (!isPIMSource(id)) {
            continue;
        }        
        CExtItem* item = customList.getItem(id);        
        const char* sync = item->getSourceConfig()->getSync();
        if (sync) {
            direction = SynctypeReg2UI(sync);
            break;
        }           
    }

    lstSyncType.SetCurSel(direction);

    // check if another sync is in progress and set the position of the sync type
    if(checkStartSync() != 0) {
        lstSyncType.EnableWindow(FALSE);
#if defined(WIN32_PLATFORM_WFSP)
        spinSyncType.EnableWindow(FALSE);
#endif
    }    
}

/**
* From the installation procedure, the default dirs are:
* briefcase: My Documents\briefcase
* note: My Documents
* picture: My Documents\My Picture
*/
void CPIMSettings::initializeSearchFolder(WindowsSyncSourceConfig* sconfig, int type) {
    
    CString directory;
    string currentSyncDir = sconfig->getSyncDir();        
    wchar_t* dir = toWideChar(currentSyncDir.c_str());

    directory = dir;
    delete [] dir;
    /*
    if (!existsDirectory(dir)) {
        LPTSTR dir = new WCHAR[512];
        SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
        directory = dir;
    } else {
        directory = dir;
    }
    delete [] dir;
    */        
    if (type == SOURCE_BRIEFCASE) {
        briefcaseFolder = directory;
        SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER, directory);  
        GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDER)->SetFont(&pathFontSize);
    } else if (type == SOURCE_NOTES) {
        notesFolder = directory;
        SetDlgItemText(IDC_NOTES_STATIC_FOLDER, directory);   
        GetDlgItem(IDC_NOTES_STATIC_FOLDER)->SetFont(&pathFontSize);
    } 
    
    
}

BOOL CPIMSettings::OnInitDialog() {
    
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_SETTINGS);
    SetWindowText(s1);
    CDialog::OnInitDialog();
            
#ifdef WIN32_PLATFORM_WFSP
    spinSyncType.SetRange(4,0);	spinSyncType.SetBuddy(&lstSyncType);
#endif

    createMenu();
    InitializeFont();    
    LoadStringResources();
 
    initializeSyncDirection();

    // Hide the dropdown listbox if the syncmode available is only 1
    if (lstSyncType.GetCount() > 1) {
        lstSyncType.ShowWindow(SW_SHOW);
        GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->ShowWindow(SW_SHOW);
        lstSyncType.SetFocus();
    }
    else {
        lstSyncType.ShowWindow(SW_HIDE);
        GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->ShowWindow(SW_HIDE);
#if defined(WIN32_PLATFORM_WFSP)
        spinSyncType.ShowWindow(SW_HIDE);
#endif
    }

    // notes
    if (existsNote) {    
        CExtItem* cb = customList.getItem(SOURCE_NOTES);
        initializeSearchFolder(cb->getSourceConfig(), SOURCE_NOTES);
        
    } else {
        GetDlgItem(IDC_NOTES_STATIC_FOLDERTOSYNC)->ShowWindow(SW_HIDE);
        GetDlgItem(IDC_NOTES_STATIC_FOLDER)->ShowWindow(SW_HIDE);
        linkNotesChoose.ShowWindow(SW_HIDE);
    }

    // briefcase
    
    if (existsBriefcase) {
        CExtItem* cb = customList.getItem(SOURCE_BRIEFCASE);
        initializeSearchFolder(cb->getSourceConfig(), SOURCE_BRIEFCASE);
        
    } else {
        GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC)->ShowWindow(SW_HIDE);
        GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDER)->ShowWindow(SW_HIDE);
        linkBriefcaseChoose.ShowWindow(SW_HIDE);     
    }   
        
    Invalidate();
    UpdateWindow();

    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}

bool CPIMSettings::changedDirectory(WindowsSyncSourceConfig* sconfig, CString folder) {
    
    bool changed = false;
    // Save notes folder dir
    if(folder == _T("")) {
        WCHAR* tmp = toWideChar(sconfig->getSyncDir().c_str());
        folder = tmp;
        if (tmp) { delete [] tmp; }
    }
    else {
        char* dir = toMultibyte(folder.GetBuffer());
        int len = strlen(dir);
        if (len > 1 && dir[len-1] == '\\') {
            dir[len-1] = NULL;
        }
        if (strcmp(dir, sconfig->getSyncDir().c_str()) ) {
            sconfig->setSyncDir(dir);                
            changed = true;
        }
        if (dir) { delete [] dir; }
    }
    return changed;

}


void CPIMSettings::OnOK()
{
    if(checkStartSync()){
        CDialog::OnOK();
    }

    int pos = lstSyncType.GetCurSel();
    
    int flag = -1;
    for (int i = 0; i < customList.getCountItem(); i++) {

        int id = customList.indexToId(i);
        if (!isPIMSource(id)) {
            continue;
        }
        CExtItem* item = customList.getItem(id); 
        const char* syncType = getSyncModeAtPos(item, pos);
        if (!syncType) continue;

        const char* sync = item->getSourceConfig()->getSync();
        if (strcmp(sync, syncType) == 0) {
            // sync mode unchanged, no need to save it
            break;
        } else {
            item->getSourceConfig()->setSync(syncType);
            flag = getDirtyFlagFromId(id);
            getRegConfig()->setClientDirty(flag);
        }        
    }  

    // handle the folder in a separate way
    bool changedBriefcase = false;
    bool changedNotes     = false;
        
    CExtItem* item = customList.getItem(SOURCE_NOTES);    
    if (item) {
        changedNotes = changedDirectory(item->getSourceConfig(),     notesFolder);
        getRegConfig()->setClientDirty(getDirtyFlagFromId(SOURCE_NOTES));
    }
    item = customList.getItem(SOURCE_BRIEFCASE);    
    if (item) {
        changedBriefcase = changedDirectory(item->getSourceConfig(), briefcaseFolder);
        getRegConfig()->setClientDirty(getDirtyFlagFromId(SOURCE_BRIEFCASE));
    }    
    
    getRegConfig()->saveDirty();
    
    if (changedBriefcase) {
        HWND hw = ::FindWindow(TEXT("ClientPushWindowBriefcase"), NULL);
        if (hw) {
            ::SendMessage(hw, WM_COMMAND, 1, 0);
        }
    }
    if (changedNotes) {
        HWND hw = ::FindWindow(TEXT("ClientPushWindowNotes"), NULL);
        if (hw) {
            ::SendMessage(hw, WM_COMMAND, 2, 0);
        }
    }    
        
    CDialog::OnOK();
}


void CPIMSettings::LoadStringResources(){
   
    CString s1;

#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_SETTINGS); 
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif        

    //
    // Sync direction listbox: it's dynamically filled reading the available
    // syncmodes from the config (of the first PIM source)
    //
    StringBuffer syncModes;
    for (int i=0; i<customList.getCountItem(); i++) {
        int id = customList.indexToId(i);
        if (!isPIMSource(id)) continue;
       
        CExtItem* item = customList.getItem(id);        
        syncModes = item->getSourceConfig()->getSyncModes();
        break;
    }

    if (syncModes.find(SYNC_MODE_TWO_WAY) != StringBuffer::npos) {
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE1);
        lstSyncType.AddString(s1);
    }
    if (syncModes.find(SYNC_MODE_ONE_WAY_FROM_SERVER) != StringBuffer::npos ||
        syncModes.find(SYNC_MODE_SMART_ONE_WAY_FROM_SERVER) != StringBuffer::npos) {
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE2);
        lstSyncType.AddString(s1);
    }
    if (syncModes.find(SYNC_MODE_ONE_WAY_FROM_CLIENT) != StringBuffer::npos ||
        syncModes.find(SYNC_MODE_SMART_ONE_WAY_FROM_CLIENT) != StringBuffer::npos) {
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTCOMM_SYNCTYPE3);
        lstSyncType.AddString(s1);
    }

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCTYPE); 
    SetDlgItemText(IDC_SETTPIM_STATIC_SYNCTYPE, s1);
    GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->SetFont(&fontBold);


    // for briefcase
    CString foldertosync;    
    foldertosync.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_BRIEFCASE);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FOLDER_TO_SYNC);
    foldertosync += s1;
    SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC, foldertosync);
    GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC)->SetFont(&fontBold);    

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHOOSE);
    linkBriefcaseChoose.setText(s1);

    // for notes
    foldertosync.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOTES);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FOLDER_TO_SYNC);
    foldertosync += s1;
    SetDlgItemText(IDC_NOTES_STATIC_FOLDERTOSYNC,foldertosync);
    GetDlgItem(IDC_NOTES_STATIC_FOLDERTOSYNC)->SetFont(&fontBold);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHOOSE);
    linkNotesChoose.setText(s1);

}

void CPIMSettings::OnBnClickedBriefcaseLinkChoose() {

    CExploreFolders wndExplore;
    wndExplore.setSourceId(SOURCE_BRIEFCASE);
    CString folder;
    if (!existsDirectory(briefcaseFolder.GetBuffer())) {
        LPTSTR dir = new WCHAR[512];
        SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
        folder = dir;
        delete [] dir;
    } else {
        folder = briefcaseFolder;
    }
    wndExplore.setFolder(folder);    

    INT_PTR result = wndExplore.DoModal();
    if (result == IDCHOOSE){
        briefcaseFolder = wndExplore.getFolder();
        SetDlgItemText(IDC_BRIEFCASE_STATIC_FOLDER, briefcaseFolder);
    }

}

void CPIMSettings::OnBnClickedNotesLinkChoose() {
    
    CExploreFolders wndExplore;
    wndExplore.setSourceId(SOURCE_NOTES);
    CString folder;
    if (!existsDirectory(notesFolder.GetBuffer())) {
        LPTSTR dir = new WCHAR[512];
        SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
        folder = dir;
        delete [] dir;
    } else {
        folder = notesFolder;
    }
    wndExplore.setFolder(folder);  
       
    INT_PTR result = wndExplore.DoModal();
    if (result == IDCHOOSE) {
        // user has chosen a new folder
        notesFolder = wndExplore.getFolder();
        SetDlgItemText(IDC_NOTES_STATIC_FOLDER, notesFolder);
    }
}


// capture pressing of 'Advanced' link
BOOL CPIMSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed = false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){            
            if(GetFocus()->GetSafeHwnd() == linkNotesChoose.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedNotesLinkChoose();
            }
            if(GetFocus()->GetSafeHwnd() == linkBriefcaseChoose.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedBriefcaseLinkChoose();
            }           
        }
        // left/right/down arrow keys
        if(((pMsg->wParam == VK_DOWN) || (pMsg->wParam == VK_UP)) && dropDownOpened == false) {
            if(processKeyEvents(pMsg->wParam)){
                bProcessed = true;
            }
        }
    }

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}

bool CPIMSettings::processKeyEvents(WPARAM key) {
    bool bProcessed = false;
    HWND wndFocus = GetFocus()->GetSafeHwnd();
       
    if(key == VK_DOWN) {

        if(wndFocus == lstSyncType.GetSafeHwnd()) {
            if (existsNote) {
                linkNotesChoose.SetFocus();
            } else if (existsBriefcase) {
                linkBriefcaseChoose.SetFocus();
            }
            return true;
        } else if(wndFocus == linkNotesChoose.GetSafeHwnd()) {
            if (existsBriefcase) {
                linkBriefcaseChoose.SetFocus();
            } else {
                lstSyncType.SetFocus();
            }
            return true;
        } else if(wndFocus == linkBriefcaseChoose.GetSafeHwnd()) {
            lstSyncType.SetFocus();
            return true;
        } 
       
    }

    if(key == VK_UP) {        
        if(wndFocus == lstSyncType.GetSafeHwnd()) {
            if (existsBriefcase) {
                linkBriefcaseChoose.SetFocus();
            } else if (existsNote) {
                linkNotesChoose.SetFocus();
            }
            return true;
        } else if(wndFocus == linkNotesChoose.GetSafeHwnd()) {
            lstSyncType.SetFocus();
            return true;
        } else if(wndFocus == linkBriefcaseChoose.GetSafeHwnd()) {
            if (existsNote) {
                linkNotesChoose.SetFocus();
            } else {
                lstSyncType.SetFocus();
            }
            return true;
        }  
        
    }

    return false;
}


HBRUSH CPIMSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}

void CPIMSettings::OnSize(UINT, int, int) {
    
    int resId = IDD_UI_PIM_OPTIONS;
    if (oldOrientation == -1) {
        DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
        oldOrientation = -2;
    }

    //
    // THis is a fixed value from the main windows screen
    //


    CString s1 ,s2;
    int y_offset = 0;
    int y_offset_from_label = 0;
    int y_offset_for_dir_label = 0;
    int orientation = -1;
    int y_size         = 18; // used for all labels   
    
    int moltiplicator = 1;
    if (CuiDlg::isVGA()) {
        moltiplicator = 2;
    }
    
    CRect clientRect;  GetClientRect(&clientRect);    
    CRect winRect;     GetWindowRect(&winRect);
    
#if defined(WIN32_PLATFORM_PSPC)
    int y_from_top_winrect = 55;
#elif defined WIN32_PLATFORM_WFSP
    int y_from_top_winrect = (int)(winRect.BottomRight().y * 0.15);
#endif

    switch(DRA::GetDisplayMode()) {        
        case DRA::Landscape: 
            {   
                y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
                y_offset = 4*moltiplicator;            // distance between the checkbox  
                y_offset_for_dir_label = 2*moltiplicator;
                orientation = 0;
            }
            break;
        
        case DRA::Square:
            {
                y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
                y_offset = 4*moltiplicator;            // distance between the checkbox  
                y_offset_for_dir_label = 2*moltiplicator;
                orientation = 1;
#if defined(WIN32_PLATFORM_PSPC)
    if (CuiDlg::isQVGASquare()) {
       y_from_top_winrect = y_from_top_winrect + 20;   
    }
#endif
            }
            break;
        default: //  also case DRA::Portrait:      
        {      
            y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
            y_offset = 8*moltiplicator;            // distance between the checkbox  
            y_offset_for_dir_label = 4*moltiplicator;
            orientation = 2;
        }    
        break;
    };   
    
    y_from_top_winrect = y_from_top_winrect * moltiplicator;
    CWnd* pWnd = NULL;

    // difference between top of windowRect and Client rect. This is used in the
    // SetWindowsPos
    int offset = winRect.BottomRight().y - clientRect.BottomRight().y;        
    
    // start info for the checkbox and draw them
    int x_start      = (int)(winRect.BottomRight().x * 0.02); 
    int y_start      = (int)(y_from_top_winrect - offset); 
    int x_stype_size = (int)(winRect.BottomRight().x * 0.40);          

    y_size = y_size * moltiplicator;
    int y_size_choose  = y_size + 4; // used for choose button   
    int y_size_static_label = y_size * 2; // used for the label with the path

    // 
    // position of the static label and the dropdown list
    // (if <=1 is hidden, [better way to know this?] so no need to reposition them)
    if (lstSyncType.GetCount() > 1) {

        pWnd = GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE);
        pWnd->SetWindowPos(GetActiveWindow(), x_start,
            y_start,
            x_stype_size,
            y_size,
            SWP_NOZORDER);

        CRect labelRec;    
        GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->GetWindowRect(&labelRec);        

#if defined(WIN32_PLATFORM_PSPC)   

        int x_dropdown_start   = x_start + x_stype_size;
        int x_dropdown_size    = (int)(winRect.BottomRight().x * 0.5);

        pWnd = GetDlgItem(IDC_PIM_SYNCTYPE);
        CRect ddrect;        
        pWnd->GetWindowRect(&ddrect);
        int y_size_drow_down_box = (ddrect.bottom - ddrect.top);

        pWnd->SetWindowPos(GetActiveWindow(), x_dropdown_start,
            y_start,
            x_dropdown_size,
            y_size_drow_down_box,
            SWP_NOZORDER);


        y_start  = labelRec.BottomRight().y - offset + y_offset_from_label;

#elif defined WIN32_PLATFORM_WFSP

        y_start  = labelRec.BottomRight().y - offset + y_offset_for_dir_label;

        int x_dropdown_start   = x_start;
        int x_dropdown_size    = (int)(winRect.BottomRight().x * 0.5);

        pWnd = GetDlgItem(IDC_PIM_SYNCTYPE);
        CRect ddrect;        
        pWnd->GetWindowRect(&ddrect);
        int y_size_drow_down_box = (ddrect.bottom - ddrect.top);

        pWnd->SetWindowPos(GetActiveWindow(), x_dropdown_start,
            y_start,
            x_dropdown_size,
            y_size_drow_down_box,
            SWP_NOZORDER);

        pWnd = GetDlgItem(IDC_SETTPIM_SPIN1);
        pWnd->SetWindowPos(GetActiveWindow(), x_dropdown_size + 2,
            y_start,
            25 * moltiplicator,
            y_size_drow_down_box,
            SWP_NOZORDER);

        y_start += (y_size_drow_down_box + y_offset_from_label); 

#endif
    }
    
    int x_folder_size    = (int)(winRect.BottomRight().x * 0.70);     
    int x_choose_size    = (int)(winRect.BottomRight().x * 0.25); 
    int x_label_size     = (int)(winRect.BottomRight().x * 0.90); 
        
    int x_choose_start   = x_start + x_folder_size;

    if (existsNote) {
        
        pWnd = GetDlgItem(IDC_NOTES_STATIC_FOLDERTOSYNC);
        pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                              y_start,
                                              x_folder_size,
                                              y_size,
                                              SWP_NOZORDER);

        
        pWnd = GetDlgItem(IDC_NOTES_LINK_CHOOSE);
        pWnd->SetWindowPos(GetActiveWindow(), x_choose_start,
                                              y_start,
                                              x_choose_size,
                                              y_size_choose,
                                              SWP_NOZORDER);

        y_start += (y_size + y_offset_for_dir_label);

        pWnd = GetDlgItem(IDC_NOTES_STATIC_FOLDER);
        pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                              y_start,
                                              x_label_size,
                                              y_size_static_label,
                                              SWP_NOZORDER);

        y_start += (y_size_static_label + y_offset);
    }
    
    if (existsBriefcase) {
        
        pWnd = GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDERTOSYNC);
        pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                              y_start,
                                              x_folder_size,
                                              y_size,
                                              SWP_NOZORDER);

        
        pWnd = GetDlgItem(IDC_BRIEFCASE_LINK_CHOOSE);
        pWnd->SetWindowPos(GetActiveWindow(), x_choose_start,
                                              y_start,
                                              x_choose_size,
                                              y_size_choose,
                                              SWP_NOZORDER);

        y_start += (y_size + y_offset_for_dir_label);

        pWnd = GetDlgItem(IDC_BRIEFCASE_STATIC_FOLDER);
        pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                              y_start,
                                              x_label_size,
                                              y_size_static_label,
                                              SWP_NOZORDER);

        y_start += (y_size_static_label + y_offset);
    }               
      
}
