// PictureSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "PictureSettings.h"
#include "uiDlg.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "ExploreFolders.h"
#include "pim/utils.h"
#include "pim/clientsettings.h"
#include "pim/Maincpp.h"

// If true, displays a sync direction STATIC label on top of the options screen
// (the syncmodes supported for pictures is only 1, so no dropdown)
#define SHOW_PICTURES_SYNC_DIRECTION    false


// PictureSettings dialog

IMPLEMENT_DYNAMIC(CPictureSettings, CDialog)

CPictureSettings::CPictureSettings(CWnd* pParent /*=NULL*/)
	: CDialog(CPictureSettings::IDD, pParent),
      customList(((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl()) {
    
    oldOrientation = -1;  
}

CPictureSettings::~CPictureSettings() { }

void CPictureSettings::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

    DDX_Control(pDX, IDC_PICTURE_LINK_CHOOSE,  linkPictureChoose);
    DDX_Control(pDX, IDC_PICTURE_CHECK_OLD_ITEMS,  checkOldItems);
    
}


BEGIN_MESSAGE_MAP(CPictureSettings, CDialog)

    ON_BN_CLICKED(IDC_PICTURE_LINK_CHOOSE, &CPictureSettings::OnBnClickedPictureLinkChoose)
    ON_WM_CTLCOLOR()
    ON_WM_SIZE()
END_MESSAGE_MAP()


bool CPictureSettings::createMenu() {   
    
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_OKCANCEL;

    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return false;
    };
    return true;    
}

void CPictureSettings::InitializeFont() { 
   
    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT)GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    GetObject(hFont, sizeof pathlf, &pathlf);
    
    pathlf.lfHeight += (2 * FONT_HEIGHT_OFFSET);
    VERIFY(pathFontSize.CreateFontIndirect(&pathlf));
       
    lf.lfHeight += FONT_HEIGHT_OFFSET;
    VERIFY(defaultFontSize.CreateFontIndirect(&lf));
     
    // add the bold
    lf.lfWeight = FW_BOLD;
    VERIFY(fontBold.CreateFontIndirect(&lf)); 
        
}

/**
* From the installation procedure, the default dirs are:
* picture: My Documents\My Picture
*/
void CPictureSettings::initializeSearchFolder(WindowsSyncSourceConfig* sconfig, int type) {
    
    CString directory;
    string currentSyncDir = sconfig->getSyncDir();        
    wchar_t* dir = toWideChar(currentSyncDir.c_str());
    
    pictureFolder = dir;
    delete [] dir;
    
    /*
    if (!existsDirectory(dir)) {
        LPTSTR dir = new WCHAR[512];
        SHGetSpecialFolderPath(NULL, dir, CSIDL_PERSONAL, FALSE);
        directory = dir;
    } else {
        directory = dir;
    }
    delete [] dir;
            
    pictureFolder = directory;
    SetDlgItemText(IDC_PICTURE_STATIC_FOLDER, directory); 
    */
    SetDlgItemText(IDC_PICTURE_STATIC_FOLDER, pictureFolder); 
    GetDlgItem(IDC_PICTURE_STATIC_FOLDER)->SetFont(&pathFontSize);
     
}

BOOL CPictureSettings::OnInitDialog() {
    
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PICTURES_SETTINGS);
    SetWindowText(s1);
    CDialog::OnInitDialog();
            
    createMenu();
    InitializeFont();    
    LoadStringResources();
       
    CExtItem* cb = customList.getItem(SOURCE_PICTURES);
    WindowsSyncSourceConfig* wssc = cb->getSourceConfig();
    if (!wssc) return FALSE;

    // Hide the static sync direction (the syncmode available is only 1)
    if (!SHOW_PICTURES_SYNC_DIRECTION) {
        GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->ShowWindow(SW_HIDE);
        GetDlgItem(IDC_PICTURE_STATIC_DIRECTION)->ShowWindow(SW_HIDE);
    }

    initializeSearchFolder(wssc, SOURCE_PICTURES);  

    if (wssc) {
        if (wssc->getIncludeOldItems()) {
            checkOldItems.SetCheck(BST_CHECKED);
        } else {
            checkOldItems.SetCheck(BST_UNCHECKED);
        }
    }
            
    Invalidate();
    UpdateWindow();

    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}

bool CPictureSettings::changedDirectory(WindowsSyncSourceConfig* sconfig, CString folder) {
    
    bool changed = false;
    // Save notes folder dir
    if(folder == _T("")) {
        WCHAR* tmp = toWideChar(sconfig->getSyncDir().c_str());
        folder = tmp;
        if (tmp) { delete [] tmp; }
    }
    else {
        char* dir = toMultibyte(folder.GetBuffer());
        int len = strlen(dir);
        if (len > 1 && dir[len-1] == '\\') {
            dir[len-1] = NULL;
        }
        if (strcmp(dir, sconfig->getSyncDir().c_str()) ) {
            sconfig->setSyncDir(dir);                
            changed = true;
        }
        if (dir) { delete [] dir; }
    }
    return changed;

}

void CPictureSettings::OnOK()
{
    if(checkStartSync()){
        CDialog::OnOK();
    }
    
    // handle the folder in a separate way
    bool changedPicture   = false;
            
    CExtItem* item = customList.getItem(SOURCE_PICTURES);    
    if (item) {
        WindowsSyncSourceConfig* wssc = item->getSourceConfig();
        if (wssc) {
            // set pictures dir
            changedPicture = changedDirectory(wssc, pictureFolder);

            // set pictures filter flag
            if(checkOldItems.GetCheck() == BST_CHECKED) {
                wssc->setIncludeOldItems(true);
            } else {
                wssc->setIncludeOldItems(false);
            }

            getRegConfig()->setClientDirty(getDirtyFlagFromId(SOURCE_PICTURES));
        }
    }
    
    getRegConfig()->saveDirty();
        
    if (changedPicture) {
        HWND hw = ::FindWindow(TEXT("ClientPushWindowPicture"), NULL);
        if (hw) {
            ::SendMessage(hw, WM_COMMAND, 3, 0);
        }
    }
        
    CDialog::OnOK();
}

void CPictureSettings::LoadStringResources(){
   
    CString s1;

#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PICTURES_SETTINGS); 
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif        
    
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCTYPE); 
    SetDlgItemText(IDC_SETTPIM_STATIC_SYNCTYPE, s1);
    GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->SetFont(&fontBold);

    CString foldertosync;
    // for picture
    foldertosync.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PICTURES);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FOLDER_TO_SYNC);
    foldertosync += s1;
    SetDlgItemText(IDC_PICTURE_STATIC_FOLDERTOSYNC,foldertosync);
    GetDlgItem(IDC_PICTURE_STATIC_FOLDERTOSYNC)->SetFont(&fontBold);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHOOSE);
    linkPictureChoose.setText(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_INCLUDE_OLDER_PICTURES);
    SetDlgItemText(IDC_PICTURE_CHECK_OLD_ITEMS, s1);
}

void CPictureSettings::OnBnClickedPictureLinkChoose() {
    CExploreFolders wndExplore;
    wndExplore.setSourceId(SOURCE_PICTURES);
     CString folder;
    if (!existsDirectory(pictureFolder.GetBuffer())) {
        LPTSTR dir = new WCHAR[512];
        SHGetSpecialFolderPath(NULL, dir, CSIDL_MYPICTURES, FALSE);
        folder = dir;
        delete [] dir;
    } else {
        folder = pictureFolder;
    }
    wndExplore.setFolder(folder);       

    INT_PTR result = wndExplore.DoModal();
    if (result == IDCHOOSE) {
        // user has chosen a new folder
        pictureFolder = wndExplore.getFolder();
        SetDlgItemText(IDC_PICTURE_STATIC_FOLDER, pictureFolder);
    }
}

HBRUSH CPictureSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}


// capture pressing of 'Advanced' link
BOOL CPictureSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed = false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){            
            if(GetFocus()->GetSafeHwnd() == linkPictureChoose.GetSafeHwnd()){
                bProcessed = true;
                OnBnClickedPictureLinkChoose();
            }                     
        }        
    }

    if(bProcessed)
        return TRUE;
    else
        return CDialog::PreTranslateMessage(pMsg);
}



void CPictureSettings::OnSize(UINT, int, int) {
    
    int resId = IDD_UI_PICTURESETTINGS;
    if (oldOrientation == -1) {
        DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
        oldOrientation = -2;
    }

    //
    // THis is a fixed value from the main windows screen
    //


    CString s1 ,s2;
    int y_offset = 0;
    int y_offset_from_label = 0;
    int y_offset_for_dir_label = 0;
    int orientation = -1;
    int y_size         = 18; // used for all labels   
    long style;
    int moltiplicator = 1;
    if (CuiDlg::isVGA()) {
        moltiplicator = 2;
    }
    
    CRect clientRect;  GetClientRect(&clientRect);    
    CRect winRect;     GetWindowRect(&winRect);
    
#if defined(WIN32_PLATFORM_PSPC)
    int y_from_top_winrect = 55;
#elif defined WIN32_PLATFORM_WFSP
    int y_from_top_winrect = (int)(winRect.BottomRight().y * 0.15);
#endif

    switch(DRA::GetDisplayMode()) {        
        case DRA::Landscape: 
            {   
                y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
                y_offset = 4*moltiplicator;            // distance between the checkbox  
                y_offset_for_dir_label = 2*moltiplicator;
                orientation = 0;
                style = ES_LEFT;
            }
            break;
        
        case DRA::Square:
            {
                y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
                y_offset = 4*moltiplicator;            // distance between the checkbox  
                y_offset_for_dir_label = 2*moltiplicator;
                orientation = 1;
                style = ES_RIGHT;
            }
            break;
        default: //  also case DRA::Portrait:      
        {      
            y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
            y_offset = 8*moltiplicator;            // distance between the checkbox  
            y_offset_for_dir_label = 4*moltiplicator;
            orientation = 2;
            style = ES_RIGHT;
        }    
        break;
    };   
    
    y_from_top_winrect = y_from_top_winrect * moltiplicator;
    CWnd* pWnd = NULL;

    // difference between top of windowRect and Client rect. This is used in the
    // SetWindowsPos
    int offset = winRect.BottomRight().y - clientRect.BottomRight().y;        
    
    // start info 
    int x_start      = (int)(winRect.BottomRight().x * 0.02); 
    int y_start      = (int)(y_from_top_winrect - offset); 
    int x_stype_size = (int)(winRect.BottomRight().x * 0.40); ;             

    y_size = y_size * moltiplicator;
    int y_size_choose  = y_size + 4; // used for choose button   
    int y_size_static_label = y_size * 3; // used for the label with the path

    // 
    // position of the static label and the label
    //    
    if (SHOW_PICTURES_SYNC_DIRECTION) {
        pWnd = GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE);
        pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                              y_start,
                                              x_stype_size,
                                              y_size,
                                              SWP_NOZORDER);

        CRect labelRec;    
        GetDlgItem(IDC_SETTPIM_STATIC_SYNCTYPE)->GetWindowRect(&labelRec);        
            
        int x_dropdown_start   = x_start + x_stype_size;
        int x_dropdown_size    = (int)(winRect.BottomRight().x * 0.5);

        pWnd = GetDlgItem(IDC_PICTURE_STATIC_DIRECTION);
        pWnd->ModifyStyle(ES_LEFT | ES_RIGHT | ES_CENTER, style);
        pWnd->SetWindowPos(GetActiveWindow(), x_dropdown_start,
                                              y_start,
                                              x_dropdown_size,
                                              y_size,
                                              SWP_NOZORDER);

        y_start  = labelRec.BottomRight().y - offset + y_offset_from_label;
    }
    
    int x_folder_size    = (int)(winRect.BottomRight().x * 0.70);     
    int x_choose_size    = (int)(winRect.BottomRight().x * 0.25); 
    int x_label_size     = (int)(winRect.BottomRight().x * 0.90); 
        
    int x_choose_start   = x_start + x_folder_size;
        
    pWnd = GetDlgItem(IDC_PICTURE_STATIC_FOLDERTOSYNC);
    pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                          y_start,
                                          x_folder_size,
                                          y_size,
                                          SWP_NOZORDER);

    pWnd = GetDlgItem(IDC_PICTURE_LINK_CHOOSE);
    pWnd->SetWindowPos(GetActiveWindow(), x_choose_start,
                                          y_start,
                                          x_choose_size,
                                          y_size_choose,
                                          SWP_NOZORDER);
    
    y_start += (y_size + y_offset_for_dir_label);

    pWnd = GetDlgItem(IDC_PICTURE_STATIC_FOLDER);
    pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                          y_start,
                                          x_label_size,
                                          y_size_static_label,
                                          SWP_NOZORDER);

    y_start += (y_size_static_label + y_offset);

    checkOldItems.SetWindowPos(GetActiveWindow(), x_start,
                                                  y_start,
                                                  x_label_size,
                                                  y_size,
                                                  SWP_NOZORDER);

    y_start += (y_size + y_offset);
}
