/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "pim/mail/MailClient2Server.h"
#include "pim/mail/MailInfo.h"
#include "pim/mail/MailItem.h"
#include "pim/mail/MailClientData.h"
#include "pim/mail/MessagingManager.h"

#include "pim/account.h"

#include "pim/utils.h"
#include "base/util/utils.h"
#include "base/util/ArrayList.h"
#include <mapidefs.h>
#include <mapicode.h>
#include <mapitags.h>
#include <mapix.h>
#include <mapiutil.h>
#include <cemapi.h>
#include "wtypes.h"
#include "base/quoted-printable.h"

#include "HwndFunctions.h"

//------------------------------------------------------------- Typedef
enum {
    eeePR_ENTRYID,
    eeePR_MESSAGE_FLAGS,
    eeePR_LAST_MODIFICATION_TIME,
    eeeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeeNUM_COLS, endSync) =
{
    eeeNUM_COLS,
    PR_ENTRYID,                       // entry id
    PR_MESSAGE_FLAGS,
    PR_LAST_MODIFICATION_TIME        //

};

enum {
    ePR_ENTRYID,
    ePR_MESSAGE_FLAGS,
    //MM
    ePR_LAST_MODIFICATION_TIME,
    ePR_SUBJECT,
    ePR_SUBJECT_PREFIX,
    ePR_MESSAGE_SIZE,
    ePR_SENDER_NAME,
    ePR_SENDER_EMAIL_ADDRESS,
    ePR_HASATTACH,
    ePR_MESSAGE_DELIVERY_TIME,
    ePR_CUSTOM_MESSAGE_ID,
    ePR_CUSTOM_MESSAGE_HEADER,
    ePR_REPLY_RECIPIENT_NAMES,
    ePR_REPLY_RECIPIENT_ENTRIES,
    ePR_CLIENT_SUBMIT_TIME,
    ePR_IMPORTANCE,
    //MM
    NUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(NUM_COLS, properties) =
{
    NUM_COLS,
    PR_ENTRYID,                         // entry id
    PR_MESSAGE_FLAGS,
    //MM
    PR_LAST_MODIFICATION_TIME,          //
    PR_SUBJECT,                         // mail Subject
    PR_SUBJECT_PREFIX,
    PR_MESSAGE_SIZE,                    // sum of the size of every property of the object
    PR_SENDER_NAME,                     // "Marco "
    PR_SENDER_EMAIL_ADDRESS,            // From  "Marco <magi@libero.it>
    PR_HASATTACH,
    PR_MESSAGE_DELIVERY_TIME,
    PR_CUSTOM_MESSAGE_ID,
    PR_CUSTOM_MESSAGE_HEADER,
    PR_REPLY_RECIPIENT_NAMES,
    PR_REPLY_RECIPIENT_ENTRIES,
    PR_CLIENT_SUBMIT_TIME,
    PR_IMPORTANCE

};

enum {
    eePR_ENTRYID,
    eePR_LAST_MODIFICATION_TIME,
    eePR_DISPLAY_NAME,
    eePR_MESSAGE_FLAGS,
    eeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeNUM_COLS, Columns) =
{
    eeNUM_COLS,
    PR_ENTRYID,
    PR_LAST_MODIFICATION_TIME,
    PR_DISPLAY_NAME,
    PR_MESSAGE_FLAGS
};

//--------------------------------------------------------- Constructors

MailClient2Server::MailClient2Server() {}

MailClient2Server::~MailClient2Server() {}


int MailClient2Server::completeObject(MailClientData& mcd, IMessage* pmsg, const bool isUpdate) {

    int ret = -1;
    StringBuffer parentRole;
    SPropValue rgprops[2]   = {0};
    ULONG    cProps         = 0;
    ULONG    cmsgValue      = 0;
    SPropValue* msgprops    = NULL;
    bool read               = false;
    char * hdrs             = NULL;

    HRESULT hr = pmsg->GetProps((LPSPropTagArray)&properties, MAPI_UNICODE, &cmsgValue, &msgprops);
    EXIT_ON_FAILED (hr);

    if (!msgprops) {
        LOG.error("completeObject: can't get message props.");
        return 1;
    }
    if (msgprops[ePR_ENTRYID].ulPropTag != PR_ENTRYID) {
        LOG.error("entry id not found.");
        return 2;
    }


    //********************* Set EmailData properties ***************************

    MailData* mailData = mcd.getEmailData();
    if (!mailData) {
        LOG.error("No email data");
        return 3;
    }

    // Reset flags (is it still needed?)
    mailData->setRead(false);
    mailData->setForwarded(false);
    mailData->setReplied(false);
    mailData->setReceived(false);
    mailData->setCreated(false);
    mailData->setDeleted(false);
    mailData->setFlagged(false);


    if (msgprops[ePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
        if (msgprops[ePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ) {
            read = true;
        }
        else {
            read = false;
        }
    }
    mailData->setRead(read);

    // If it's an update, and the folder is not draft, do not set the message body
    // from 7.1.2 also for the slow sync and the 'I' it is an update
    parentRole = mcd.getParentRole();
    if (isUpdate && parentRole != ROLE_DRAFT && parentRole != ROLE_OUTBOX){
        LOG.debug("Update only");

        MailMessage& msg = mailData->getEmailItem();

        msg.getBody().setContent("");
        char *msgId = NULL;
        if (msgprops[ePR_CUSTOM_MESSAGE_ID].ulPropTag == PR_CUSTOM_MESSAGE_ID) {
            msgId = toMultibyte(msgprops[ePR_CUSTOM_MESSAGE_ID].Value.lpszW);
            if (!msgId) {
                LOG.error("completeObject: error retrieving messageId from POOM");
            }
        }
        msg.setMessageId(msgId);
        delete [] msgId;
    }
    else {
        /*************************************************************************
         * Past this point, the whole message is sent to the server, and
         * must be complete.
         *************************************************************************/
        FILETIME ft;

        wchar_t time[20];

        if (parentRole == ROLE_TRASH && msgprops[ePR_MESSAGE_DELIVERY_TIME].ulPropTag == PR_MESSAGE_DELIVERY_TIME){
            ft = msgprops[ePR_MESSAGE_DELIVERY_TIME].Value.ft;  //OK
        }
        else if (msgprops[ePR_LAST_MODIFICATION_TIME].ulPropTag == PR_LAST_MODIFICATION_TIME){
            ft = msgprops[ePR_LAST_MODIFICATION_TIME].Value.ft;  //OK
        }
        else {
            // Should never happen
            LOG.error("Error retrieving last modification time");
            LOG.error("Using current time");
            SYSTEMTIME st;
            GetLocalTime(&st);
            SystemTimeToFileTime(&st, &ft);
        }
        // Set date in the Email header.
        FileTime2UTCTimeFormatter(ft, time);
        char *t = toMultibyte(time);
        mailData->setModified(t);
        delete [] t;

        //*************** Set emailItem props *******************************/
        MailMessage& msg = mailData->getEmailItem();
        ULONG attachCount = 0;

        // Set date ------------------------------------------------------------
        BasicTime bDate;

        // In case of Outbox we use always the current time
        if (parentRole == ROLE_OUTBOX) {
            SYSTEMTIME systemTime;
            GetSystemTime(&systemTime);
            SystemTimeToFileTime(&systemTime, &ft);
        }

        FileTimeToBasicDate(ft, &bDate);
        msg.setDate(bDate);

        // Set subject ------------------------------------------------------------
        if (msgprops[ePR_SUBJECT].ulPropTag == PR_SUBJECT) {
            msg.setSubject(_wcc(msgprops[ePR_SUBJECT].Value.lpszW));
        }

        // priority -> PR_IMPORTANCE. PR_PRIORITY is not a field valid
        long importance = 0;
        msg.setImportance("3"); // IMPORTANCE_NORMAL is default

        if (msgprops[ePR_IMPORTANCE].ulPropTag == PR_IMPORTANCE) {
            importance = msgprops[ePR_IMPORTANCE].Value.l;
            if (importance == IMPORTANCE_HIGH) {
                msg.setImportance("1"); // high
            } else if (importance == IMPORTANCE_LOW) {
                msg.setImportance("5"); // low
            }
        }

        wchar_t* emailSender = NULL;
        wchar_t* sender      = NULL;
        wchar_t* accountName = NULL;
        wchar_t* fr          = NULL;

        // Set sender address -----------------------------------------------------
        if (msgprops[ePR_SENDER_EMAIL_ADDRESS].ulPropTag == PR_SENDER_EMAIL_ADDRESS) {
            LOG.debug("MailClientToServer: PR_SENDER_EMAIL_ADDRESS: %ls", msgprops[ePR_SENDER_EMAIL_ADDRESS].Value.lpszW);

            emailSender = msgprops[ePR_SENDER_EMAIL_ADDRESS].Value.lpszW;

            if (msgprops[ePR_SENDER_NAME].ulPropTag == PR_SENDER_NAME) {
                sender = msgprops[ePR_SENDER_NAME].Value.lpszW;
            }

            if (sender != NULL && wcscmp(sender, emailSender) == 0 && parentRole == ROLE_OUTBOX) {
                if (wcscmp(emailSender, DEFAULT_USER_W) == 0) {
                    msg.setFrom("");
                } else {
                    StringBuffer fromHdr;
                    const char* sender = toMultibyte(emailSender);


                    if (sender == NULL) {
                        msg.setFrom("");
                        LOG.error("%s: error setting email sender in message header", __FUNCTION__);
                    } else {
                        MessagingManager* msgMan = getMessagingManager();
                        MailAccount* account = msgMan->getAccountFromMailAddr(sender);
                        
                        if (account != NULL) {
                            const char* visibleName = account->getVisibleName();
                            if (visibleName) {
                                fromHdr.sprintf("\"%s\" <%s>", visibleName, sender);
                            } else {
                                fromHdr = sender;
                            }
                        } else {
                            LOG.info("%s: The account from email address is not found. Set default From:", __FUNCTION__);                            
                            fromHdr = "";
                        }
                        delete [] sender;
                    }

                    msg.setFrom(fromHdr.c_str());
                }
            } else {
                // if the email is default user it means the sender is empty and the user
                // want to use the default mail added by the server.
                // The check wcsstr of the email sender is on the acute only to avoid
                // something like default@email.com that can be valid
                if (wcscmp(emailSender, DEFAULT_USER_W) == 0 ||
                    wcsstr(emailSender, DEFAULT_USER_W_ACUTE) != 0) {
                    msg.setFrom("");
                } else {
                    msg.setFrom(_wcc(emailSender));
                }
            }
            //msg.setFrom(_wcc(emailSender));
            LOG.debug("MailClientToServer: From: %s\n", msg.getFrom());
        }

        // Set recipients ---------------------------------------------------------
        char* to = NULL, *cc = NULL, *bcc = NULL;
        hr = GetRecipients(pmsg, &to, &cc, &bcc);
        EXIT_ON_FAILED (hr);

        msg.setTo (to);
        msg.setCc (cc);
        msg.setBcc(bcc);

        if (to) delete [] to;
        if (cc) delete [] cc;
        if (bcc) delete [] bcc;

        // Set body -----------------------------------------------------------
        BodyPart bodyPart;
        wchar_t *buffer = NULL;
        char* body = NULL;
        hr = GetBody(pmsg, &buffer);
        EXIT_ON_FAILED (hr);

        body = toMultibyte(buffer);
        delete [] buffer;

        if (!body) {
            LOG.error("null body for message %s", msg.getSubject());
            ret = 4;
            goto FuncExit;
        }

        bodyPart.setContent(body);
        msg.setBody(bodyPart);
        delete [] body;

        // Set attachments ----------------------------------------------------
        if (msgprops[ePR_HASATTACH].ulPropTag == PR_HASATTACH) {
            attachCount = msgprops[ePR_HASATTACH].Value.ul;
            GetAttachments(pmsg, &mcd);
        }

        // Message Id ---------------------------------------------------------
        char *msgId = NULL;
        if (msgprops[ePR_CUSTOM_MESSAGE_ID].ulPropTag == PR_CUSTOM_MESSAGE_ID) {
            //msgId = toMultibyte(msgprops[ePR_CUSTOM_MESSAGE_ID].Value.lpszW);
            msgId = toMultibyte( (MessagingManager::createKeyFromId( msgprops[ePR_CUSTOM_MESSAGE_ID].Value.bin )).c_str() );
            if (!msgId) {
                LOG.error("completeObject: error retrieving messageId from POOM");
            }
        }
        else {
            msgId = createMessageID(mcd.getEntryId());
            WCHAR* id = toWideChar(msgId);
            if (id) {
                // Set message id in POOM
                rgprops[cProps].ulPropTag = PR_CUSTOM_MESSAGE_ID;
                rgprops[cProps].Value.lpszW  = id;
                ++cProps;

                rgprops[cProps].ulPropTag = PR_LAST_MODIFICATION_TIME;
                rgprops[cProps].Value.ft  = ft;
                ++cProps;

                delete [] id;
            }
            else {
                LOG.error("completeObject: error converting message id: %s", id);
            }

        }
        // Set message id in outgoing message
        msg.setMessageId(msgId);
        delete [] msgId;

        // Other headers --------------------------------------------------------
        if (msgprops[ePR_CUSTOM_MESSAGE_HEADER].ulPropTag ==
            PR_CUSTOM_MESSAGE_HEADER) {
            hdrs = toMultibyte(msgprops[ePR_CUSTOM_MESSAGE_HEADER].Value.lpszW);
            if (hdrs) {
                msg.setHeaders(hdrs);
            }
            else {
                LOG.error("completeObject: error retrieving headers.");
            }
        }

        if (cProps > 0) {
            HRESULT hhr = pmsg->SetProps(cProps, rgprops, NULL);
            int k = 0;
        }
    }
    ret = 0;    // ok

FuncExit:
    if (hdrs) {delete [] hdrs; hdrs = NULL; }
    return ret;
}

BOOL openTmpFile(wchar_t* filename, FILE** f) {

    BOOL ret = TRUE;
    *f = _wfopen(filename, TEXT("w+b"));

    if(f == NULL)
        ret = FALSE;

    return ret;
}

void closeTmpFile(FILE* f) {

    if (f) {
        fflush(f);
        fclose(f);
    }
}

int writeToTmpFile(FILE* f, char *ptr, int numBytes) {

   int numWritten = fwrite(ptr, sizeof(char), numBytes, f);
   return numWritten;

}

HRESULT MailClient2Server::GetAttachments(IMessage* pmsg, MailClientData* mcd) {

    LOG.debug("Enter into GetAttachments function");
    LPMAPITABLE pAttachTable = NULL;
    // for each row in the attach table
    LPSRowSet pRows = NULL;
    int currentRow = 0;
    char szBuf[2048];
    ULONG ulNumChars;

    HRESULT hr = pmsg->GetAttachmentTable(0, &pAttachTable);
    EXIT_ON_FAILED(hr);

    // iterate all the attachment
    while(TRUE) {

        hr = pAttachTable->QueryRows(1, 0, &pRows);
        if (FAILED(hr) || pRows->cRows != 1)
            break;
        currentRow++;

        // only one row is returned anyway
        int nProps = pRows->aRow[0].cValues;
        long num = -1;
        long size = -1;
        wchar_t* filename = NULL, *filenameTmp = NULL;
        long msgstatus = -1;
        IAttach* attach;

        for (int i=0; i<nProps; i++) {
            ULONG ulPT = pRows->aRow[0].lpProps[i].ulPropTag;
            // loop through all the properties

            // TCHAR* szPropName = ReverseLookup(ulPT);
            switch (ulPT) {
                case PR_ATTACH_NUM:
                    num = pRows->aRow[0].lpProps[i].Value.ul;
                    break;
                case PR_ATTACH_SIZE:
                    size = pRows->aRow[0].lpProps[i].Value.ul;
                    break;
                case PR_ATTACH_FILENAME:
                    filenameTmp = pRows->aRow[0].lpProps[i].Value.lpszW;
                    if (filenameTmp)
                        filename = wstrdup(filenameTmp);
                    break;
                case PR_MSG_STATUS:
                    msgstatus = pRows->aRow[0].lpProps[i].Value.ul;
                    break;
                case 0x30000003:
                    // pRows->aRow[0].lpProps[ii].Value.ul: sequence number
                    break;
                default:
                    break;
            }    // switch

        }
        FreeProws(pRows);

        pmsg->OpenAttach(num, NULL, 0, &attach);
        // read IStream->Read way to write into an array of char.
        IStream* pStream;
        hr = attach->OpenProperty(PR_ATTACH_DATA_BIN, 0, NULL, ATTACH_BY_VALUE, (LPUNKNOWN *)&pStream);
        EXIT_ON_FAILED(hr);

        wchar_t tmpPath[64];
        wchar_t tmpFileName[MAX_PATH];  // System constants for the path

        GetTempPath(64, tmpPath);
        GetTempFileName(tmpPath, TEXT("fun"), 0, tmpFileName);

        FILE* f = NULL;
        f = _wfopen(tmpFileName, TEXT("w+b"));

        if (f == NULL)
            goto FuncExit;

        int m = 0;
        do {
            pStream->Read(szBuf, 2048, &ulNumChars);
            writeToTmpFile(f, szBuf, ulNumChars);
            m = m + ulNumChars;

        } while (ulNumChars >= 2048);

        closeTmpFile(f);

        pStream->Release();

        BodyPart bodyPart;
        bodyPart.setFilename(_wcc(filename));
        bodyPart.setContent(_wcc(tmpFileName));
        bodyPart.setEncoding("base64"); // currently the ppc default

        if (wcsstr(filename, TEXT(".wav"))) {
            bodyPart.setMimeType("audio/x-wav");
        } else if (wcsstr(filename, TEXT(".bmp"))) {
            bodyPart.setMimeType("image/x-ms-bmp");
        } else if (wcsstr(filename, TEXT(".gif"))) {
            bodyPart.setMimeType("image/gif");
        } else if (wcsstr(filename, TEXT(".jpg"))) {
            bodyPart.setMimeType("image/jpg");
        } else if (wcsstr(filename, TEXT(".psw"))) {
            bodyPart.setMimeType("application/msword");
        } else if (wcsstr(filename, TEXT(".htm"))) {
            bodyPart.setMimeType("text/html");
        }

        mcd->getEmailData()->getEmailItem().addAttachment(bodyPart);

        if (filename)
            delete [] filename;

    }
FuncExit:
    // Free each element in the pRows buffer
    FreeProws(pRows);
    RELEASE_OBJ(pAttachTable);
    LOG.debug("Exiting from GetAttachments function");
    return hr;

}


HRESULT MailClient2Server::GetBody(IMessage* pmsg, wchar_t** body) {

    LOG.debug("Enter into GetBody function");
    IStream* pStream;
    ULONG ulNumChars;
    ULONG ulPos = 0;
    HRESULT    hr;
    ULARGE_INTEGER plibNewPosition, startPosition;
    LARGE_INTEGER dlibMove;
    dlibMove.QuadPart= 0;

    // Open the Stream to the body
    hr = pmsg->OpenProperty(PR_BODY, NULL, STGM_READ, NULL,(LPUNKNOWN *) &pStream);
    EXIT_ON_FAILED(hr);
    pStream->Seek( dlibMove, STREAM_SEEK_END, &plibNewPosition);

    *body = new wchar_t[(size_t)plibNewPosition.QuadPart + 1];
    memset(*body, 0, ((size_t)plibNewPosition.QuadPart + 1)* sizeof(wchar_t));

    pStream->Seek( dlibMove, STREAM_SEEK_SET, &startPosition);

    hr = pStream->Read(*body, (size_t)plibNewPosition.QuadPart, &ulNumChars);
    EXIT_ON_FAILED(hr);

    pStream->Release();

    return S_OK;


FuncExit:
    LOG.error("Error reading body mail");
    return -1;
}

wstring BuildAddress(IN LPTSTR pszDisplayName, IN LPTSTR pszEmailAddress)
{
    LOG.debug("Enter into BuildAddress function");
    wstring addressBuilt;
    if (pszDisplayName == NULL && pszEmailAddress !=NULL ) {
        addressBuilt.append(pszEmailAddress);
    } else if (pszDisplayName!=NULL && pszEmailAddress==NULL) {

 		wstring encodedDisplayName;
 		encodedDisplayName.append(TEXT("=?utf-8?Q?"));
        char* displayName = toMultibyte(pszDisplayName);
        char* encoded = qp_encode(displayName);
        const WCHAR* wencoded = toWideChar(encoded);
        encodedDisplayName.append(wencoded);
        delete [] encoded;
        delete [] wencoded;
        delete [] displayName;
 		encodedDisplayName.append(TEXT("?="));

        addressBuilt.append(TEXT("\""));
        addressBuilt.append(encodedDisplayName);
        addressBuilt.append(TEXT("\""));
    } else if (pszDisplayName!=NULL && pszEmailAddress!=NULL) {
        addressBuilt.append(TEXT("\""));
        addressBuilt.append(pszDisplayName);
        addressBuilt.append(TEXT("\""));
        addressBuilt.append(TEXT(" <"));
        addressBuilt.append(pszEmailAddress);
        addressBuilt.append(TEXT(">"));
    } else {
        addressBuilt.append(TEXT("UNKNOWN"));
    }
    LOG.debug("Exiting from BuildAddress function");
    return addressBuilt;
}

// concatenate the recipients part
HRESULT MailClient2Server::GetRecipients(LPMESSAGE pMessage, char** to, char** cc, char** bcc) {

    LOG.debug("Enter into GetRecipients function");
    // for the recipient table
    LPMAPITABLE pRecipientTable = NULL;
    // for each row in the recipient table
    LPSRowSet pRows = NULL;
    int currentRow  = 0;
    int nTempLen    = 0;
    wstring::iterator it;

    wstring szTempTO, szTempCC, szTempBCC;

    // pRecipientTable->SetColumn does not work. Returning result is sth. like "NOT IMPLEMENTED".
    // loop through all rows without limiting the columns
    HRESULT hr = pMessage->GetRecipientTable(0, &pRecipientTable);
    EXIT_ON_FAILED(hr);

    // iterate all the recipients
    while(TRUE) {

        hr = pRecipientTable->QueryRows(1, 0, &pRows);
        if (FAILED(hr) || pRows->cRows != 1) break;
        currentRow++;

        // only one row is returned anyway
        int nProps = pRows->aRow[0].cValues;

        TCHAR *szReceipName, *szReceipEmail;
        ULONG ulReceipType;
        szReceipEmail = NULL;
        szReceipName = NULL;
        ulReceipType = 0;

        // loop through all the properties
        for (int ii=0; ii<nProps; ii++) {
            ULONG ulPT = pRows->aRow[0].lpProps[ii].ulPropTag;

            // TCHAR* szPropName = ReverseLookup(ulPT);
            switch (ulPT) {
            case PR_EMAIL_ADDRESS:
                szReceipEmail = pRows->aRow[0].lpProps[ii].Value.LPSZ;
                break;
            case PR_RECIPIENT_TYPE:
                ulReceipType = pRows->aRow[0].lpProps[ii].Value.ul;
                break;
            case PR_DISPLAY_NAME:
                szReceipName = pRows->aRow[0].lpProps[ii].Value.LPSZ;
                break;
            case PR_ADDRTYPE:
                // pRows->aRow[0].lpProps[ii].Value.LPSZ. Always "SMTP"
                break;
            case 0x30000003:
                // pRows->aRow[0].lpProps[ii].Value.ul: sequence number
                break;
            default:
                break;
            }    // switch
        }

        // now let us build the String
        wstring szAddr = BuildAddress(szReceipName, szReceipEmail);
        if (ulReceipType==MAPI_TO) {
            szTempTO.append(szAddr);
            szTempTO.append(TEXT(","));
        } else if (ulReceipType==MAPI_CC) {
            szTempCC.append(szAddr);
            szTempCC.append(TEXT(","));
        } else if (ulReceipType==MAPI_BCC) {
            szTempBCC.append(szAddr);
            szTempBCC.append(TEXT(","));
        }

        // Free each element in the pRows buffer
        FreeProws(pRows);
    }    // end of the loop through all recipients

    // remove the trailing ",", if any
    int nLen = 0;

    nLen = szTempTO.size();
    if (nLen > 0) {
        it = szTempTO.end() -1;
        if (*it == L',')
            *it = L'\0';
    }

    nLen = szTempCC.size();
    if (nLen > 0) {
        it = szTempCC.end() -1;
        if (*it == L',')
            *it = L'\0';
    }

    nLen = szTempBCC.size();
    if (nLen > 0) {
        it = szTempBCC.end() -1;
        if (*it == L',')
            *it = L'\0';
    }

    nTempLen = szTempTO.size();
    if (nTempLen>0) {
        *to = toMultibyte(szTempTO.c_str());
    }
    nTempLen = szTempCC.size();
    if (nTempLen>0) {
        *cc = toMultibyte(szTempCC.c_str());
    }
    nTempLen = szTempBCC.size();
    if (nTempLen>0) {
        *bcc = toMultibyte(szTempBCC.c_str());
    }

FuncExit:
    // Free each element in the pRows buffer
    FreeProws(pRows);
    RELEASE_OBJ(pRecipientTable);
    LOG.debug("Exiting from GetRecipients function");
    return hr;
}

HRESULT MailClient2Server::markAsRead(IMessage* pMsg) {

    SPropValue  rgprops[3]  = {0};
    ULONG       cProps      = 0;
    ULONG       cmsgValue   =  0;
    SPropValue* msgprops    = NULL;
    ULONG       messageFlag = (ULONG)0x00000000;
    FILETIME    ft;


    HRESULT hr = pMsg->GetProps((LPSPropTagArray)&properties, MAPI_UNICODE, &cmsgValue, &msgprops);
    EXIT_ON_FAILED (hr);

    if (msgprops[ePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
        messageFlag = msgprops[ePR_MESSAGE_FLAGS].Value.ul;
    }

    if (msgprops[ePR_LAST_MODIFICATION_TIME].ulPropTag == PR_LAST_MODIFICATION_TIME) {
        ft = msgprops[ePR_LAST_MODIFICATION_TIME].Value.ft;
    }

    // Set the flags
    rgprops[cProps].ulPropTag = PR_MESSAGE_FLAGS;
    rgprops[cProps].Value.ul = messageFlag | MSGFLAG_READ | MSGFLAG_FROMME;
    ++cProps;

    rgprops[cProps].ulPropTag = PR_CLIENT_SUBMIT_TIME;
    rgprops[cProps].Value.ft = ft;
    ++cProps;

    rgprops[cProps].ulPropTag = PR_MESSAGE_DELIVERY_TIME;
    rgprops[cProps].Value.ft = ft;
    ++cProps;

    // Save the props
    hr = pMsg->SetProps(cProps, rgprops, NULL);
    EXIT_ON_FAILED(hr);

FuncExit:
    return hr;
}



MailClientData* MailClient2Server::createMailForSendLog(const StringBuffer& content) {

    MailClientData* mcd = new MailClientData(TEXT(""));

    mcd->getEmailData()->setRead(false);
    mcd->getEmailData()->setForwarded(false);
    mcd->getEmailData()->setReplied(false);
    mcd->getEmailData()->setReceived(false);
    mcd->getEmailData()->setCreated(false);
    mcd->getEmailData()->setDeleted(false);
    mcd->getEmailData()->setFlagged(false);

    MailMessage& msg = mcd->getEmailData()->getEmailItem();

    // This date doesn't matter at all for the Outbox.
    // it needs only the go over the addMessage check
    BasicTime bDate;
    SYSTEMTIME systemTime;
    FILETIME ft;
    GetSystemTime(&systemTime);
    SystemTimeToFileTime(&systemTime, &ft);
    FileTimeToBasicDate(ft, &bDate);
    msg.setDate(bDate);

    msg.setSubject("[Report] Synclog");

    char tt[64];
    if (strstr(FUNOPT_SEND_LOG_ADDRESS, "@") != 0) {
        sprintf(tt, "%s", FUNOPT_SEND_LOG_ADDRESS);
    } else {
        sprintf(tt, "%s%s%s", FUNOPT_SEND_LOG_ADDRESS, "@", "funambol.com");
    }

    msg.setTo(tt);

    BodyPart bodyPart;
    //bodyPart.setContent("This is the log of the user");
    StringBuffer bb("Client configuration: \r\n\r\n");
    bb += content;
    bb += " \r\n Follows the log of the user\r\n";
    bodyPart.setContent(bb);
    msg.setBody(bodyPart);

    BodyPart attach;

    WCHAR tmpPath[64];
    WCHAR tmpFileName[MAX_PATH];  // System constants for the path
    GetTempPath(64, tmpPath);
    wsprintf(tmpFileName, TEXT("%s\\synclog.txt"), tmpPath);

    char buffer[16384];
    DWORD MAX_LOG_SIZE = 500000;

    HANDLE hfile = CreateFile(TEXT("\\synclog.txt"), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    HANDLE hfilew = CreateFile(tmpFileName, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);

    DWORD dwSize = 0, lpFileSizeHigh = 0;
    dwSize = GetFileSize(hfile, &lpFileSizeHigh);
    if (dwSize == 0xFFFFFFFF) {
        dwSize = 0;
    }

    if (dwSize > MAX_LOG_SIZE) {
        DWORD ss = SetFilePointer(hfile, dwSize - MAX_LOG_SIZE, 0, FILE_BEGIN);
    }

    DWORD nBytesRead = 0;
    DWORD nBytesWrote = 0;
    while(hfile && hfilew) {
        BOOL bResult = ReadFile(hfile, &buffer, 16384, &nBytesRead, NULL);
        // Check for the end of file.

        if (bResult &&  nBytesRead == 0) {
            CloseHandle(hfile);
            CloseHandle(hfilew);
            break;
        } else {
            WriteFile(hfilew, buffer, nBytesRead, &nBytesWrote, NULL);
        }
    }

    // Add the log as attachment
    attach.setFilename("synclog.txt");
    const char* ctmp = toMultibyte(tmpFileName);
    attach.setContent(ctmp);
    delete [] ctmp;
    attach.setEncoding("base64"); // currently the ppc default

    mcd->getEmailData()->getEmailItem().addAttachment(attach);

    return mcd;
}
