/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef __CUSTOMIZATION_H
#define __CUSTOMIZATION_H

//
// IMPORTANT: Don't change the indentation of this file!!!
//

// Set this to change the place where the configuration is stored
// in the registry, starting from HKLM/Software
#define ROOT_CONTEXT                    "Funambol/SyncclientPIM"
#define ROOT_CONTEXT_W                  TEXT("Funambol\\SyncclientPIM")
#define SOURCES_CONTEXT                 ROOT_CONTEXT "/spds/sources"

// Set this to the name of the manufacturer
#define PROVIDER                        "Funambol"

// This is the name of the link (lnk) in the Start Menu. Used to start the service after reboot
#define APP_NAME                        PROVIDER

// Default values for the SyncML account created in backward compatibility mode
#define DEFAULT_ACCOUNT_NAME             TEXT(PROVIDER)
#define DEFAULT_EMAIL_ADDRESS            "default@email"
#define DEFAULT_VISIBLE_NAME             PROVIDER

// The name of the transport service used by the application. For the Funambol
// client the name is the provider. For other could be more generically "SyncML"
// It is used by the handleservice in the wap-provisioningdoc and in the registry.cpp
// to set the wm registry
#define SERVICE_TYPE                    "SyncML"

// URL showed in the About box.
#define APP_LINK                        "www.funambol.com"

// Domain of funambol Email account (optional)
#define DOMAIN_NAME                     L"funambol.com"

// The incoming/outgoing Email Server
#define INCOMING_MAIL_SERVER            L"sync.funambol.com"
#define OUTGOING_MAIL_SERVER            L"sync.funambol.com"

#define FUNAMBOL_SIGNATURE "funambol :: mobile open source :: http://www.funambol.com"

// Conditional defines for various customizable features.

// If set to 1, hides the URL in the communication settings.
#define FUNOPT_SHOW_URL                 1
#define FUNOPT_SHOW_DEV_INFO            0   // show the device info in about. it hides FUNOPT_SHOW_COPYRIGHT and FUNOPT_SHOW_ABOUT_MAIN_WEB_SITE
#define FUNOPT_SHOW_COPYRIGHT           1
#define FUNOPT_SHOW_ABOUT_MAIN_WEB_SITE 1

// it define if to show the license gpl3 or the Powered by Funambol\nwww.funambol.com
#define FUNOPT_SHOW_LICENSE             1

// it defines if to show the send log menu and which is the address to send the log (default to funambol)
#define FUNOPT_SHOW_SEND_LOG            1
#define FUNOPT_SEND_LOG_ADDRESS         "fwm-logs"

// pictures sync
#define MAX_IMAGE_SIZE                  50000   /** maximum size in KBytes of a single photo item: 50MB (so it's like we don't have a limit) */
#define ENABLE_PICTURES_CLEANUP         0       /** if 1, will show the cleanup of pictures in cleanup dialog (0 is default) */

// If set to 1, the login is automatically started when the Account screen is displeyed (no user interaction)
#define FUNOPT_AUTO_LOGIN               0
// Auto-update feature
#define UP_URL_RESOURCE                 "/sapi/profile/client?action=get-update-info"

#ifdef WIN32_PLATFORM_WFSP
    #define CLIENT_PLATFORM             "wmsph"
#endif
#ifdef WIN32_PLATFORM_PSPC
    #define CLIENT_PLATFORM             "wmppc"
#endif

// Bitmask to set which choices the user has in the push method drop down list
// 1 - manual
// 2 - push
// 4 - scheduled
// it is possible to set a combination of these values
// 1+2+4 = 7 - show all the manual, push, scheduled options
#define FUNOPT_SHOW_PUSH_METHOD_LIST    7     

//
//
// Set how the push method has to start
// the options are 
// 1 - manual
// 2 - push
// {15, 30, 60, 120, 240, 360, 480, 720, 1440} - minutes of scheduling
#define FUNOPT_START_PUSH_METHOD_OPTION 2 

#define FUNOPT_ENABLE_C2S_PUSH      1
#define FUNOPT_ENABLE_S2C_PUSH      1
#define FUNOPT_ENABLE_SMS_PUSH      1


// List of available sync modes for each source (comma separated values).
// These are the values available from the client's settings for each source.
#define PIM_SYNC_MODES                  SYNC_MODE_TWO_WAY
#define MAIL_SYNC_MODES                 SYNC_MODE_TWO_WAY
#define PICTURES_SYNC_MODES             SYNC_MODE_SMART_ONE_WAY_FROM_CLIENT

// Default sync mode for each source.
// It MUST be one of the values specified in the list of available sync modes above.
#define DEFAULT_PIM_SYNC_MODE           SYNC_MODE_TWO_WAY
#define DEFAULT_MAIL_SYNC_MODE          SYNC_MODE_TWO_WAY
#define DEFAULT_PICTURES_SYNC_MODE      SYNC_MODE_SMART_ONE_WAY_FROM_CLIENT

#endif
