/*
* Funambol is a mobile platform developed by Funambol, Inc.
* Copyright (C) 2003 - 2007 Funambol, Inc.
*
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU Affero General Public License version 3 as published by
* the Free Software Foundation with the addition of the following permission
* added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
* WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
* WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
*
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
* details.
*
* You should have received a copy of the GNU Affero General Public License
* along with this program; if not, see http://www.gnu.org/licenses or write to
* the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
* MA 02110-1301 USA.
*
* You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
* 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
*
* The interactive user interfaces in modified source and object code versions
* of this program must display Appropriate Legal Notices, as required under
* Section 5 of the GNU Affero General Public License version 3.
*
* In accordance with Section 7(b) of the GNU Affero General Public License
* version 3, these Appropriate Legal Notices must retain the display of the
* "Powered by Funambol" logo. If the display of the logo is not reasonably
* feasible for technical reasons, the Appropriate Legal Notices must display
* the words "Powered by Funambol".
*/

#ifndef INCL_ACCOUNT_MANAGER
#define INCL_ACCOUNT_MANAGER
/** @cond DEV */

#include "base/util/utils.h"
#include "base/util/ArrayElement.h"
#include "base/util/StringBuffer.h"
#include "mail/MailSyncSourceConfig.h"
#include "mail/MailAccount.h"
#include "spds/FolderData.h"
#include "base/globalsdef.h"
#include "mail/MailAccountManager.h"
#include "base/fscapi.h"

#include <string>
using namespace std;

BEGIN_NAMESPACE

class AccountProvisioner {

public:

    AccountProvisioner(){};
    ~AccountProvisioner(){};

    /**
    * Creates the email account on the Client.
    * @param account      the account settings informations
    * @param accountGuid  [OUT] the account GUID new generated
    * @return 0 if no errors
    */
    int addAccount(MailAccount& account, wstring& accountGuid);

    /**
    * Updates an email account.
    * @param account the account settings informations
    * @param accountGuid the account GUID to be updated
    * @return 0 if no errors, -1 if account not found
    */
    int modAccount(const MailAccount& account, const WCHAR* accountGuid);

    /**
    * Deletes an email account.
    * @param accountGuid the account GUID to be removed
    * @return 0 if no errors, -1 if account not found
    */
    int delAccount(const WCHAR* accountGuid);


    /**
    * Returns the guid as for Microsoft specification
    */
    wstring formatGUID(GUID* pGuid) const;


private:

    /**
    * Creates and returns a unique GUID as for Microsoft specification
    */
    wstring createGUID() const;

};

END_NAMESPACE
#endif
