/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef INCL_MAIL_SYNC_SOURCE
#define INCL_MAIL_SYNC_SOURCE
/** @cond WMPLUGIN_MAINCLIENT */
/** @{*/


#include "base/fscapi.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/SyncMap.h"
#include "spds/SyncStatus.h"
#include "spds/SyncReport.h"
#include "base/util/ArrayElement.h"
#include "pim/mail/MailClient2Server.h"
#include "pim/mail/MailServer2Client.h"
#include "pim/utils.h"
#include "pim/mail/MailUtils.h"
#include "pim/BaseSyncSource.h"
#include "pim/mail/MessagingManager.h"

#include "events/SyncSourceListenerServer.h"
#include "event/SetListener.h"

#define MAIL_SYNC_SOURCE_LISTENER_NAME "mailSyncSourceListener"

class MessagingStatus;          // mail status cache
class SyncSourceListenerServer; // listener for server start/end sync notifications

__declspec(dllexport) class MailSyncSource : public SyncSource {

private:
    /*
    * It is the path in which the application is.
    */
    wchar_t* path;

    SyncItem* syncItem;

    bool isMailInOutbox;
    bool isMailInInbox;
    bool failedSendMailInOutbox;

    /// The manager of email accounts/folders/messages.
    MessagingManager messagingManager;

    /// List of all items keys (list of ItemData objects)
    ArrayList allItems;

    ArrayList newItems;
    ArrayList modItems;
    ArrayList delItems;

    /// Counter of ALL items (SLOW SYNC)
    int iAll;

    /// Counter of NEW items
    int iNew;

    /// Counter of MOD items
    int iMod;

    /// Counter of DEL items
    int iDel;
    

    /*
    * The max mail message. Used in the inclusive behaviour
    */
    int maxMailMessageSize;
    bool isSyncInclusive;

    // to store if the user filter includes the attachments
    bool includeAttachment;

    // server support for MEA (from syncMl capabilities)
    int serverHasMeaCap;

    // MessagingStatus singleton instance 
    MessagingStatus*        msgStatusCache;
    static MailSyncSource*  pinstance;

    // listener for sync source server notifications
    SyncSourceListenerServer* syncSourceListener;

    /**
     * Parses the email data inside item.getData() and set it into the MailClientData object.
     * Sets additional params into the MailClientData object, like the parent folder key and maxMessageSize.
     * @param item      the input SynItem to parse
     * @param m         [OUT] the MailClientData object to fill with data
     * @param isUpdate  true if it's an updated item, false if it's a new item
     * @return          integer value specified by mailItemStatus enumeration 
     */
    int  parseMailData(SyncItem &item, MailClientData &m, bool isUpdate);


public:

    __declspec(dllexport) static MailSyncSource* getInstance(const wchar_t* name, MailSyncSourceConfig* sc);

    /**
     * Constructor: create a SyncSource with the specified name
     *
     * @param name - the name of the SyncSource
     */
    MailSyncSource(const wchar_t* name, MailSyncSourceConfig* sc);

    /**
     * Copy Constructor: create a copy of the specified source
     * @param s - a reference to the other source
     */
    MailSyncSource(MailSyncSource &s);

    /**
     * Destructor
     */
    ~MailSyncSource();


    void setPath(const wchar_t* p);
    const wchar_t* getPath();

    // TODO: remove
    void setIsMailInOutbox(bool value);
    bool getIsMailInOutbox();
    void setIsMailInInbox(bool value);
    bool getIsMailInInbox();
    void setFailedSendMailInOutbox(bool value);
    bool getFailedSendMailInOutbox();
    //---

    void setMaxMailMessageSize(int value) { maxMailMessageSize = value; }
    int getMaxMailMessageSize() { return maxMailMessageSize;}

    void setIsSyncInclusive(bool value) { isSyncInclusive = value; }
    bool getIsSyncInclusive() { return isSyncInclusive;}

    
    void setIncludeAttachment(bool v) { includeAttachment = v; }
    bool getIncludeAttachment() { return includeAttachment; }

    SyncItem* buildSyncItem(MailClientData* m);
    SyncItem* buildSyncItem(ItemData* item);
    SyncItem* buildSyncItem(const wchar_t* accountId);

    /*
     * Return the first SyncItem of all.
     * It is used in case of slow or refresh sync
     * and retrieve the entire data source content.
     */
    SyncItem* getFirstItem();

    /*
     * Return the next SyncItem of all.
     * It is used in case of slow or refresh sync
     * and retrieve the entire data source content.
     */
    SyncItem* getNextItem();

    /*
     * Return the first SyncItem of new one. It is used in case of fast sync
     * and retrieve the new data source content.
     */
    SyncItem* getFirstNewItem();

    /*
     * Return the next SyncItem of new one. It is used in case of fast sync
     * and retrieve the new data source content.
     */
    SyncItem* getNextNewItem();

    /*
     * Return the first SyncItem of updated one. It is used in case of fast sync
     * and retrieve the new data source content.
     */
    SyncItem* getFirstUpdatedItem() ;

    /*
     * Return the next SyncItem of updated one. It is used in case of fast sync
     * and retrieve the new data source content.
     */
    SyncItem* getNextUpdatedItem();

    /*
     * Return the first SyncItem of updated one. It is used in case of fast sync
     * and retrieve the new data source content.
     */
    SyncItem* getFirstDeletedItem();

    /*
     * Return the next SyncItem of updated one. It is used in case of fast sync
     * and retrieve the new data source content.
     */

    SyncItem* getNextDeletedItem();

    void setItemStatus(const WCHAR* key, int status);

    int addItem(SyncItem& item);

    int updateItem(SyncItem& item);

    int deleteItem(SyncItem& item);

    int beginSync();

    int endSync();

    MailSyncSource& operator=(MailSyncSource& s);
   
    int removeAllItems();

    MessagingManager& getMsgManager() { return messagingManager; }


    /**
     * Used for backward compatibility with old Servers (< v8).
     * Returns the parent folder's key from the item's key in "old-style" format
     * like "I/<b64-key>". "I" stands for "inbox".
     * The default account is used.
     * @param itemKey  the email key in the format "I/<b64-key>"
     * @return         the parent folder's key (b64 format)
     */
    wstring getParentFolderFromItemKey(const WCHAR* itemKey);

    enum {
        MAIL_ITEM_NO_ERROR    = 0,
        MAIL_ITEM_PARSE_ERROR = 1,
        MAIL_ITEM_STORE_ERROR
    } mailItemStatus;

};

__declspec(dllexport) MailSyncSource* getMailSyncSource(const wchar_t* name, MailSyncSourceConfig* sc);

/** @}*/
/** @endcond */
#endif
