/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef INCL_MAIL_UTILS
    #define INCL_MAIL_UTILS

#include "base/util/ArrayElement.h"
#include "base/util/ArrayList.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "client/MailSourceManagementNode.h"
#include "notify/checknet.h"
#include "base/util/BasicTime.h"
#include "customization.h"

#include <mapidefs.h>
#include <cemapi.h>

//
// Custom propertis to store messageID and extra headers
//
#define PR_CUSTOM_MESSAGE_ID        PROP_TAG(PT_TSTRING, 0x8010)
#define PR_CUSTOM_MESSAGE_HEADER    PROP_TAG(PT_TSTRING, 0x8020)

#define NO_ACTION_MAIL     0
#define NEW_MAIL           1
#define UPDATED_MAIL       2

#define EXIT_ON_FAILED(_hr)   \
  if (FAILED(_hr))  {  \
    LOG.error("FAILED(%x) at %hs:%d\n", _hr, __FILE__, __LINE__); \
    goto FuncExit; \
  }

#ifndef RELEASE_OBJ
#define RELEASE_OBJ(s) \
  if (s != NULL)   \
  {          \
    s->Release();  \
    s = NULL;    \
  }
#endif //RELEASE_OBJ

// server backward compatibility
#define MEA_SERVER_VERSION_ID       80200 // server version 8.2
#define INBOX_KEY                   "I"
#define OUTBOX_KEY                  "O"
#define INBOX_KEY_W                 TEXT(INBOX_KEY)
#define OUTBOX_KEY_W                TEXT(OUTBOX_KEY)
#define INBOX_MESSAGE_KEY_PREFIX    "I/"
#define OUTBOX_MESSAGE_KEY_PREFIX   "O/"
#define INBOX_MESSAGE_KEY_WPREFIX   TEXT(INBOX_MESSAGE_KEY_PREFIX)
#define OUTBOX_MESSAGE_KEY_WPREFIX  TEXT(OUTBOX_MESSAGE_KEY_PREFIX)
#define SERVER_ROOT_FOLDER          "/"
#define SERVER_ROOT_FOLDER_W        TEXT(SERVER_ROOT_FOLDER)
#define SERVER_INBOX_KEY            "INBOX"
#define SERVER_OUTBOX_KEY           "Outbox"
#define SERVER_INBOX_PARENT_KEY     "ROOT/I"
#define SERVER_INBOX_PARENT_KEY_W   TEXT("ROOT/I")
#define SERVER_OUTBOX_PARENT_KEY    "ROOT/O"
#define SERVER_OUTBOX_PARENT_KEY_W  TEXT("ROOT/O")

wchar_t* convertBinaryToWChar(SBinary sbEntry, wchar_t folder);
char* convertBinaryToChar(SBinary sbEntry, wchar_t folder);
char* createMessageID(const wchar_t* entryID);
void FileTimeToBasicDate(FILETIME ft, BasicTime *b);

#endif
