/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/extensions/HelperMacros.h>
#include "base/fscapi.h"
#include "base/util/utils.h"
#include "pim/mail/MailSyncSource.h"
#include "mail/MailAccount.h"
#include "pim/mail/MessagingManager.h"

USE_NAMESPACE

#define ROOT_CONTEXT        "Funambol/SyncclientPIM"
#define SOURCES_CONTEXT     ROOT_CONTEXT "/spds/sources"

#define TEST_ACCOUNT_NAME   "Test account"
#define NEW_VISIBLE_NAME    "New name"
#define MOD_VISIBLE_NAME    "Modified name"
#define NEW_EMAIL_ADDRESS   "newemail@email.com"
#define MOD_EMAIL_ADDRESS   "modemail@email.com"

#define DATA_NEW_ACCOUNT    "<Folder><name>" TEST_ACCOUNT_NAME "</name><created>20090428T162654Z</created><role>account</role><Ext><XNam>VisibleName</XNam> <XVal>" NEW_VISIBLE_NAME "</XVal></Ext><Ext><XNam>EmailAddress</XNam> <XVal>" NEW_EMAIL_ADDRESS "</XVal></Ext></Folder>"
#define DATA_MOD_ACCOUNT    "<Folder><name>" TEST_ACCOUNT_NAME "</name><created>20090428T162654Z</created><role>account</role><Ext><XNam>VisibleName</XNam> <XVal>" MOD_VISIBLE_NAME "</XVal></Ext><Ext><XNam>EmailAddress</XNam> <XVal>" MOD_EMAIL_ADDRESS "</XVal></Ext></Folder>"

#define DATA_INBOX_FOLDER   "<Folder><name>Inbox</name><created>20090428T162654Z</created><role>inbox</role></Folder>"
#define DATA_OUTBOX_FOLDER  "<Folder><name>Outbox</name><created>20090428T162654Z</created><role>outbox</role></Folder>"

#define TEST_FOLDER_ACCOUNT_NAME1       "Test folder1"
#define TEST_FOLDER_ACCOUNT_NAME2       "Test folder2"

#define TEST_EMAIL_ACCOUNT_NAME         "Test Email"
#define TEST_EMAIL_ACCOUNT_ADDR         "default@email.com"

#define TEST_EMAIL_RECIPIENT            "test@email.com"
#define TEST_EMAIL_BODY                 "This is the mail body.\n"

#define TEST_EMAIL_ACCOUNT_NAME1        "first account"
#define TEST_EMAIL_ACCOUNT_NAME2        "second account"

#define DATA_NEW_EMAIL      "<Email>" \
                            "<read>false</read> <forwarded>false</forwarded> <replied>false</replied>" \
                            "<created>20090423T073147Z</created>" \
                            "<modified>20090423T073147Z</modified>" \
                            "<deleted>false</deleted>" \
                            "<flagged>false</flagged>" \
                            "<emailitem>" \
                            "<![CDATA[Received: by 192.168.1.19 with SMTP id x19cs44999fgb;\n" \
                            "Thu, 23 Apr 2009 00:42:38 -0700 (PDT)\n" \
                            "Date: Thu, 23 Apr 2009 09:31:47 +0200 (CEST)\n" \
                            "From: Newsletter\n" \
                            "Reply-To: noreply@newsletter.com\n" \
                            "To: Name.Surname@gmail.com\n" \
                            "Message-ID: <g081de34g0000000kb71a662mnidn.4320.16.35.28.h2435569.0._.magnews@smtpted>\n" \
                            "Subject: %s\n" \
                            "MIME-Version: 1.0\n" \
                            "Content-Type: text/plain; charset=UTF-8\n" \
                            "Content-Transfer-Encoding: 8bit\n" \
                            "\n" \
                            "This is the mail body.\n" \
                            "\n" \
                            "]]>" \
                            "</emailitem>" \
                            "</Email>"



class MailSyncSourceTest : public CppUnit::TestFixture {
    CPPUNIT_TEST_SUITE(MailSyncSourceTest);
 
    CPPUNIT_TEST(testMailSyncSourceAddAccount);
    CPPUNIT_TEST(testMailSyncSourceModAccount);
    CPPUNIT_TEST(testMailSyncSourceDelAccount);

    CPPUNIT_TEST(testMailSyncSourceFolder);
    CPPUNIT_TEST(testMailSyncSourceWrongFolder);
    
    CPPUNIT_TEST(testMailSyncSourceEmail_1);
    CPPUNIT_TEST(testMailSyncSourceEmail_2);
    CPPUNIT_TEST(testMailSyncSourceOutbox);

    CPPUNIT_TEST(testMailSyncSourceGetParentFolderFromItemKey);

    // test for Messaging status cache interaction
    CPPUNIT_TEST(testMailSyncSourceGetItemsFromCache);

    CPPUNIT_TEST(testCleanUp);
    
    CPPUNIT_TEST_SUITE_END();

public:

    void setUp() {

        // Set the MailSyncSource
        m = new MailSourceManagementNode(SOURCES_CONTEXT, "mails");
        MailSyncSourceConfig& sc = m->getMailSourceConfig(true);
        getRegConfig()->setMailSyncSourceConfig(sc);
        source = getMailSyncSource(SOURCE_NAME_MAIL_W, &sc);
    }

    void tearDown() {
        if (m) {
            delete m;
        }
        if (source) {
            delete source;
        }
    }

private:

    MailSourceManagementNode* m;
    MailSyncSource* source;

    WString accountLUID;

    void testCleanUp() {

        addAccount(TEST_EMAIL_ACCOUNT_NAME1);
        addAccount(TEST_EMAIL_ACCOUNT_NAME2);

        wstring itemKey1 = testAddEmail(TEST_EMAIL_ACCOUNT_NAME1, "Email subject 1");
        wstring itemKey2 = testAddEmail(TEST_EMAIL_ACCOUNT_NAME1, "Email subject 2");
        wstring itemKey3 = testAddEmail(TEST_EMAIL_ACCOUNT_NAME2, "Email subject 3");

        MessagingManager& msgMan = source->getMsgManager();
        getRegConfig()->setMailSyncSourceConfig(msgMan.getMailSyncSourceConfig());
        getRegConfig()->save();

        MailSyncSourceConfig& mssc = msgMan.getMailSyncSourceConfig();

      //  mssc.setToBeCleanedFlag(TEST_EMAIL_ACCOUNT_NAME1, true);

   //     ArrayList al = mssc.getMailAccounts();
   //     for (int i = 0; i < al.size(); i++){
   //         MailAccount * ma = (MailAccount*)al[i];
   //         if(strcmp(ma->getName(), TEST_EMAIL_ACCOUNT_NAME1) == 0 ){
   //             CPPUNIT_ASSERT(ma->getToBeCleaned());
   //         }        
   //     }
        mssc.setLast(1);
        getRegConfig()->setMailSyncSourceConfig(msgMan.getMailSyncSourceConfig());
        getRegConfig()->save();

        //int ret = source->removeAllItems();

//        CPPUNIT_ASSERT_MESSAGE( "Cleanup Test", ret == 0 );

     //   deleteAccount(TEST_EMAIL_ACCOUNT_NAME1);
     //   deleteAccount(TEST_EMAIL_ACCOUNT_NAME2);
        getRegConfig()->setMailSyncSourceConfig(msgMan.getMailSyncSourceConfig());
        getRegConfig()->save();

    }

    //
    // ----------------------- Tests on accounts add/update/remove ----------------------------
    //
    void testMailSyncSourceAddAccount() {

        refreshConfig();
        
        // Create the SyncItem with a NEW EMAIL ACCOUNT
        SyncItem newAccount(TEXT(""));
        StringBuffer data(DATA_NEW_ACCOUNT);
        newAccount.setData((const void*)data.c_str(), data.length());
        newAccount.setTargetParent(TEXT(""));   // empty parent

        int ret = source->addItem(newAccount);
		CPPUNIT_ASSERT(ret == STC_ITEM_ADDED);

        accountLUID = newAccount.getKey();
        CPPUNIT_ASSERT(accountLUID.empty() != true);

        // Check the config
        bool found = false;
        MailSyncSourceConfig& sc = (MailSyncSourceConfig&)source->getConfig();
        const ArrayList& mailAccounts = sc.getMailAccounts();
        for (int i=0; i<mailAccounts.size(); i++) {
            
            MailAccount* account = (MailAccount*)mailAccounts[i];
            if (account && (account->getName() == TEST_ACCOUNT_NAME)) {
                found = true;

                StringBuffer emailAddress = account->getEmailAddress();
                CPPUNIT_ASSERT(emailAddress == NEW_EMAIL_ADDRESS);
                
                StringBuffer visibleName = account->getVisibleName();
                CPPUNIT_ASSERT(visibleName == NEW_VISIBLE_NAME);
                break;
            }
		}
        CPPUNIT_ASSERT(found == true);
    }

    void testMailSyncSourceModAccount() {

        // Create the SyncItem with a MOD EMAIL ACCOUNT for the same account key
        SyncItem modAccount(accountLUID.c_str());
        StringBuffer data(DATA_MOD_ACCOUNT);
        modAccount.setData((const void*)data.c_str(), data.length());
        modAccount.setTargetParent(TEXT(""));   // empty parent

        int ret = source->updateItem(modAccount);
        CPPUNIT_ASSERT(ret == STC_OK);

        // Check the config
        bool found = false;
        MailSyncSourceConfig& sc = (MailSyncSourceConfig&)source->getConfig();
        const ArrayList& mailAccounts = sc.getMailAccounts();
        for (int i=0; i<mailAccounts.size(); i++) {
            
            MailAccount* account = (MailAccount*)mailAccounts[i];
            if (account && (account->getName() == TEST_ACCOUNT_NAME)) {
                found = true;

                StringBuffer emailAddress = account->getEmailAddress();
                CPPUNIT_ASSERT(emailAddress == MOD_EMAIL_ADDRESS);
                
                StringBuffer visibleName = account->getVisibleName();
                CPPUNIT_ASSERT(visibleName == MOD_VISIBLE_NAME);
                break;
            }
		}
        CPPUNIT_ASSERT(found == true);
    }


    void testMailSyncSourceDelAccount() {

        // Create the SyncItem (empty) with just the same account key
        SyncItem delAccount(accountLUID.c_str());
        delAccount.setTargetParent(TEXT(""));   // empty parent

        int ret = source->deleteItem(delAccount);
        CPPUNIT_ASSERT(ret == STC_OK);

        // Check the config: the account should be marked as deleted
        bool found = false;
        MailSyncSourceConfig& sc = (MailSyncSourceConfig&)source->getConfig();
        const ArrayList& mailAccounts = sc.getMailAccounts();
        for (int i=0; i<mailAccounts.size(); i++) {
            
            MailAccount* account = (MailAccount*)mailAccounts[i];
            if (account && (account->getName() == TEST_ACCOUNT_NAME) && (account->getDeleted() == false)) {
                found = true;
                break;
            }
		}
        CPPUNIT_ASSERT(found == false);
    }


    //
    // ----------------------- Tests on folders add/update/remove ----------------------------
    //

    /// Test the creation/update/deletion of inbox and outbox folder.
    void testMailSyncSourceFolder() {

        refreshConfig();

        StringBuffer accountName(TEST_FOLDER_ACCOUNT_NAME1);
        StringBuffer role;
        wstring folderID;

        addAccount(accountName);
        
        role = ROLE_INBOX;
        folderID = testAddFolder(accountName, role);
        testModFolder(accountName, folderID, role);
        testDelFolder(folderID);

        role = ROLE_OUTBOX;
        folderID = testAddFolder(accountName, role);
        testModFolder(accountName, folderID, role);
        testDelFolder(folderID);

        deleteAccount(accountName);
    }


    /// Test the creation of a wrong folder (wrong parent account, wrong role).
    void testMailSyncSourceWrongFolder() {

        refreshConfig();
        StringBuffer accountName(TEST_FOLDER_ACCOUNT_NAME2);
        wstring accountID = addAccount(accountName);

        SyncItem newFolder(TEXT(""));
        StringBuffer data(DATA_INBOX_FOLDER);

        //
        // Test wrong parent account
        //
        newFolder.setData((const void*)data.c_str(), data.length());
        newFolder.setTargetParent(TEXT("wrong-parent-key"));
        
        int ret = source->addItem(newFolder);
        CPPUNIT_ASSERT_MESSAGE("addItem succeded with wrong parent account", ret == STC_COMMAND_FAILED);
        wstring folderKey = newFolder.getKey();
        CPPUNIT_ASSERT_MESSAGE("key returned is not empty", folderKey.empty() == true);

        //
        // Test wrong folder's role
        //
        data.replaceAll(ROLE_INBOX, "wrong-role");
        newFolder.setData((const void*)data.c_str(), data.length());
        newFolder.setTargetParent(accountID.c_str());

        ret = source->addItem(newFolder);
        CPPUNIT_ASSERT_MESSAGE("addItem succeded with wrong folder role", ret == STC_COMMAND_FAILED);
        folderKey = newFolder.getKey();
        CPPUNIT_ASSERT_MESSAGE("key returned is not empty", folderKey.empty() == true);

        deleteAccount(accountName);
    }


    /// Adds a folder, given the parent account name and the folder's role.
    wstring testAddFolder(const StringBuffer& accountName, const StringBuffer& folderRole) {

        MessagingManager& msgMan = source->getMsgManager();

        StringBuffer id = msgMan.getIdOfAccount(accountName);
        wstring folderID = msgMan.getIdOfFolder(id, folderRole);    // should be the same returned by addItem
        const WCHAR* accountID = toWideChar(id);

        SyncItem newFolder(TEXT(""));
        StringBuffer data;
        if (folderRole == ROLE_INBOX)       { data = DATA_INBOX_FOLDER;  }
        else if (folderRole == ROLE_OUTBOX) { data = DATA_OUTBOX_FOLDER; }

        newFolder.setData((const void*)data.c_str(), data.length());
        newFolder.setTargetParent(accountID);

        int ret = source->addItem(newFolder);
        CPPUNIT_ASSERT_MESSAGE("testAddFolder: addItem failed", ret == STC_ITEM_ADDED);

        wstring folderKey = newFolder.getKey();
        CPPUNIT_ASSERT_MESSAGE("testAddFolder: key returned is wrong", folderKey == folderID);

        if (accountID) { delete [] accountID; }
        return folderKey;
    }

    /// Updates a folder, given the parent account name and the folder's id + role.
    void testModFolder(const StringBuffer& accountName, const wstring& folderID, const StringBuffer& folderRole) {

        MessagingManager& msgMan = source->getMsgManager();

        SyncItem modFolder(folderID.c_str());
        StringBuffer data;
        if (folderRole == ROLE_INBOX)       { data = DATA_INBOX_FOLDER;  }
        else if (folderRole == ROLE_OUTBOX) { data = DATA_OUTBOX_FOLDER; }
        modFolder.setData((const void*)data.c_str(), data.length());

        StringBuffer id = msgMan.getIdOfAccount(accountName);
        const WCHAR* accountID = toWideChar(id);
        modFolder.setTargetParent(accountID);

        int ret = source->updateItem(modFolder);
        CPPUNIT_ASSERT_MESSAGE("testModFolder: updateItem failed", ret == STC_OK);

        if (accountID) { delete [] accountID; }
    }

    /// Delete a folder, given the folder's id.
    void testDelFolder(const wstring& folderID) {

        SyncItem delFolder(folderID.c_str());   // empty data

        int ret = source->deleteItem(delFolder);
        CPPUNIT_ASSERT_MESSAGE("testDelFolder: deleteItem failed", ret == STC_OK);
    }


    //
    // ----------------------- Tests on email add/update/remove ----------------------------
    //

    /// Tests the add/mod/del of an email item
    void testMailSyncSourceEmail_1() {

        refreshConfig();
        addAccount(TEST_EMAIL_ACCOUNT_NAME);
        
        wstring itemKey = testAddEmail(TEST_EMAIL_ACCOUNT_NAME, "This is the subject");
        testModEmail(itemKey, TEST_EMAIL_ACCOUNT_NAME, "This is the subject modified");
        testDelEmail(itemKey);

        deleteAccount(TEST_EMAIL_ACCOUNT_NAME);
    }

    /// Create and modify more emails in 2 accounts
    void testMailSyncSourceEmail_2() {

        refreshConfig();
        addAccount(TEST_EMAIL_ACCOUNT_NAME1);
        addAccount(TEST_EMAIL_ACCOUNT_NAME2);
        
        wstring itemKey1 = testAddEmail(TEST_EMAIL_ACCOUNT_NAME1, "Email subject 1");
        wstring itemKey2 = testAddEmail(TEST_EMAIL_ACCOUNT_NAME1, "Email subject 2");
        wstring itemKey3 = testAddEmail(TEST_EMAIL_ACCOUNT_NAME2, "Email subject 3");
        
        testModEmail(itemKey1, TEST_EMAIL_ACCOUNT_NAME1, "Email subject 1 modified");
        testModEmail(itemKey3, TEST_EMAIL_ACCOUNT_NAME2, "Email subject 3 modified");

        testDelEmail(itemKey1);

        deleteAccount(TEST_EMAIL_ACCOUNT_NAME1);
        deleteAccount(TEST_EMAIL_ACCOUNT_NAME2);
    }

    /// Tests the add/mod/del of an email item in the outbox folder
    void testMailSyncSourceOutbox() {

        refreshConfig();
        StringBuffer accountName(TEST_EMAIL_ACCOUNT_NAME);
        addAccount(accountName);
        
        wstring itemKey = testAddEmail(accountName, "Outbox email", ROLE_OUTBOX);
        testModEmail(itemKey, accountName, "Outbox email modified");
        testDelEmail(itemKey);

        deleteAccount(accountName);
    }


    /// Adds a new email, returns the email ID (key)
    wstring testAddEmail(const StringBuffer& accountName, const StringBuffer& subject, 
                         const StringBuffer& folderRole = ROLE_INBOX) {

        MessagingManager& msgMan = source->getMsgManager();

        // Get the ID of inbox folder
        StringBuffer accountID = msgMan.getIdOfAccount(accountName);
        CPPUNIT_ASSERT_MESSAGE("error getting account ID", accountID.empty() == false);

        wstring folderID = msgMan.getIdOfFolder(accountID, folderRole);
        CPPUNIT_ASSERT_MESSAGE("error getting folder ID", folderID.size() > 0);

        // Create the SyncItem with a NEW EMAIL MESSAGE
        SyncItem newEmail(TEXT(""));
        StringBuffer data;
        data.sprintf(DATA_NEW_EMAIL, subject.c_str());

        newEmail.setData((const void*)data.c_str(), data.length());
        newEmail.setTargetParent(folderID.c_str());     // parent folder

        int ret = source->addItem(newEmail);
        CPPUNIT_ASSERT_MESSAGE("testAddEmail: addItem failed", ret == STC_ITEM_ADDED);

        wstring itemKey = newEmail.getKey();
        CPPUNIT_ASSERT_MESSAGE("testAddEmail: empty key returned", itemKey.empty() == false);
        return itemKey;
    }

    const wstring addEmailToOutbox(const StringBuffer& accountName, const StringBuffer& from,
                                   const StringBuffer& to, const StringBuffer& subject) {
        int ret = 0;
        MessagingManager& msgMan = source->getMsgManager();

         // Get the ID of account
        StringBuffer accountID = msgMan.getIdOfAccount(accountName);

        // set up mail data
        MailClientData* mcd = getMailClientData(TEXT(""));

        mcd->getEmailData()->setRead(false);
        mcd->getEmailData()->setForwarded(false);
        mcd->getEmailData()->setReplied(false);
        mcd->getEmailData()->setReceived(false);
        
        // get mail message reference
        MailData* mailData = mcd->getEmailData();
        MailMessage& msg = mailData->getEmailItem();
       
        // set up mail headers and body
        msg.setSubject(subject);    
        msg.setFrom(from);
        msg.setTo(to);

        // message date header is needed by MailServer2Client::SetMessageProps()
        BasicTime msgDate;        
        SYSTEMTIME sysTime;
        FILETIME fileTime;
        GetSystemTime(&sysTime);
        SystemTimeToFileTime(&sysTime, &fileTime);    
        FileTimeToBasicDate(fileTime, &msgDate);
        msg.setDate(msgDate);
    
        BodyPart mailBody;    
        mailBody.setContent(TEST_EMAIL_BODY);
        msg.setBody(mailBody);

        wstring folderID = msgMan.getIdOfFolder(accountID, ROLE_OUTBOX);
        CPPUNIT_ASSERT_MESSAGE("error getting folder ID", folderID.size() > 0);
        // create message in outbox folder
        mcd->setParentFolder(folderID.c_str());
        mcd->setParentRole(ROLE_OUTBOX);
        

        // add email
        ret = msgMan.addEmail(mcd);
        CPPUNIT_ASSERT_MESSAGE("failed to add message to outbox folder", ret == 0);

        // return ID of message
        const wstring msgId = mcd->getEntryId();
        LOG.info("%s: entry id: %S", __FUNCTION__, msgId.c_str());
        
        return msgId;
    }

    /// Updates an email item, given its ID
    void testModEmail(const wstring& itemKey, const StringBuffer& accountName, 
                      const StringBuffer& subject, const StringBuffer& folderRole = ROLE_INBOX) {

        MessagingManager& msgMan = source->getMsgManager();

        // Get the ID of inbox folder
        StringBuffer accountID = msgMan.getIdOfAccount(accountName);
        wstring folderID = msgMan.getIdOfFolder(accountID, folderRole);

        // Create the SyncItem with a UPDATED EMAIL MESSAGE
        SyncItem modEmail(itemKey.c_str());
        StringBuffer data;
        data.sprintf(DATA_NEW_EMAIL, subject.c_str());

        data.replace("<read>false</read>",      "<read>true</read>");
        data.replace("This is the mail body.",  "This is the mail body modified!");

        modEmail.setData((const void*)data.c_str(), data.length());
        modEmail.setTargetParent(folderID.c_str());     // parent folder

        int ret = source->updateItem(modEmail);
        CPPUNIT_ASSERT_MESSAGE("testModEmail: updateItem failed", ret == STC_OK);
    }


    /// Deletes an email item, given its ID
    void testDelEmail(const wstring& itemKey) {

        MessagingManager& msgMan = source->getMsgManager();

        // Create the SyncItem (empty, just the key to remove)
        SyncItem delEmail(itemKey.c_str());

        int ret = source->deleteItem(delEmail);
        CPPUNIT_ASSERT_MESSAGE("testDelEmail: deleteItem failed", ret == STC_OK);
    }

    // -------------------- misc tests of MailSyncSource --------------------------

    // NOTE: this test is done only if a "Funambol" account is not already existing
    void testMailSyncSourceGetParentFolderFromItemKey() {

        refreshConfig();
        MessagingManager& msgMan = source->getMsgManager();
        
        // defined in customization.h
        StringBuffer defaultName;
        defaultName.convert(DEFAULT_ACCOUNT_NAME);

        StringBuffer id = msgMan.getIdOfAccount(defaultName);

        if (id.empty()) {
            // Default account not existing: create it and test
            MailAccount account;
            account.setName(defaultName);
            account.setVisibleName(defaultName);
            account.setEmailAddress("default@email.com");

            int ret = msgMan.createAccount(account);
            CPPUNIT_ASSERT(ret == 0);

            // Get the ID of inbox folder
            StringBuffer accountID;
            accountID.convert(account.getID());
            wstring folderID = msgMan.getIdOfFolder(accountID, ROLE_INBOX);

            // Set the item key in the old style: "I/<b64-key>"
            wstring itemKey(TEXT("I/"));
            itemKey.append(TEXT("fakeB64KeyOfEmailItem"));

            // Should return the inbox folder ID of default account
            wstring parentFolder = source->getParentFolderFromItemKey(itemKey.c_str());
            CPPUNIT_ASSERT(parentFolder == folderID);

            // Delete default account
            msgMan.deleteAccount(account.getID());
        }
    }

    /* ----- MailSyncSource, MessagingStatus and MailListener integration test:
     * adding new mail messages to an account, MessagingStatus cache is updated 
     * by MailListener (tmail.exe listener) which is read by MailSyncSource 
     * when a fast sync starts
     */
    void testMailSyncSourceGetItemsFromCache() {
        int ret = 0;

        refreshConfig();
        MessagingManager& msgMan = source->getMsgManager();

        addAccount(TEST_EMAIL_ACCOUNT_NAME);
        
        // add some new mail 
        wstring messageId1 = addEmailToOutbox(TEST_EMAIL_ACCOUNT_NAME, TEST_EMAIL_ACCOUNT_ADDR, 
                                              TEST_EMAIL_RECIPIENT, "This is the subject"); 
        wstring messageId2 = addEmailToOutbox(TEST_EMAIL_ACCOUNT_NAME, TEST_EMAIL_ACCOUNT_ADDR, 
                                              TEST_EMAIL_RECIPIENT, "This is another subject");
        wstring messageId3 = addEmailToOutbox(TEST_EMAIL_ACCOUNT_NAME, TEST_EMAIL_ACCOUNT_ADDR, 
                                              TEST_EMAIL_RECIPIENT, "This is yet another subject!");

        // set a sync mode different from SYNC_SLOW otherwise mail items 
        // won't be read from MessagingStatus cache
        source->setSyncMode(SYNC_ONE_WAY_FROM_CLIENT);

        ret = source->beginSync();
        CPPUNIT_ASSERT_MESSAGE("error preparing sync", ret == 0);

        // check that the number and id of new items got from  MessagingStatus 
        // cache is coherent

        CPPUNIT_ASSERT_MESSAGE("id of first item in cache don't match", checkItemId(messageId1, true, false) == true);        
        CPPUNIT_ASSERT_MESSAGE("id of second item in cache don't match", checkItemId(messageId2, false, false) == true);        
        CPPUNIT_ASSERT_MESSAGE("id of third item in cache don't match", checkItemId(messageId3, false, false) == true);        
    
        // check if we the number of items in cache is consistent
        SyncItem* item = source->getNextNewItem();
        CPPUNIT_ASSERT_MESSAGE("invalid number of items in cache", item == NULL);

        // delete messages
        testDelEmail(messageId1);
        testDelEmail(messageId2);
        testDelEmail(messageId3);

        // remove account
        deleteAccount(TEST_EMAIL_ACCOUNT_NAME);
    }

    // ------------------- utils methods -------------------------
    /// Adds a new email account, with a specific name. Returns the new account ID.
    wstring addAccount(const StringBuffer& accountName) {

        MessagingManager& msgMan = source->getMsgManager();

        // Create the test account
        MailAccount account;
        account.setName        (accountName);
        account.setVisibleName (accountName);
        account.setEmailAddress(TEST_EMAIL_ACCOUNT_ADDR);

        int ret = msgMan.createAccount(account);
        CPPUNIT_ASSERT_MESSAGE("Failed to create account", ret == 0);
        return account.getID();
    }

    /// Deletes an email account, with a specific name.
    void deleteAccount(const StringBuffer& accountName) {

        MessagingManager* msgMan = getMessagingManager();
        StringBuffer accountID = msgMan->getIdOfAccount(accountName);
        const WCHAR* waccountID = toWideChar(accountID.c_str());

        int ret = msgMan->deleteAccount(waccountID);
        CPPUNIT_ASSERT_MESSAGE("Failed to delete account", ret == 0);

        if (waccountID) {
            delete [] waccountID;
        }
    }

    bool checkItemId(const wstring& id, bool first_search, bool updated) {
        bool ret = false;
        SyncItem* item = NULL;
        const WCHAR* itemId = NULL;
        
        if (first_search) {
            if (updated) {
                item = source->getFirstUpdatedItem();    
                
            } else {
                item = source->getFirstNewItem();
            }
        } else {
            if (updated) {
                item = source->getNextUpdatedItem();
            } else {
                item = source->getNextNewItem();    
            }
        }

        CPPUNIT_ASSERT_MESSAGE("error getting item from cache", item != NULL);

        if (item) {
            itemId = item->getKey();
            CPPUNIT_ASSERT_MESSAGE("missing ID in sync item", itemId != NULL);
            // check if item match 
            if (itemId != NULL) {
                if (wcscmp(id.c_str(), itemId) == 0) {
                    return true;
                }
            }
        }

        return ret;
    }

    /// Necessary to read the mail accounts again from registry
    void refreshConfig() {
        MailSyncSourceConfig& msc = m->getMailSourceConfig(true);
        m->setMailSourceConfig(msc);
    }

};

CPPUNIT_TEST_SUITE_REGISTRATION( MailSyncSourceTest );
