/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
// Popup.cpp : implementation file
//

#include "startsync.h"
#include "Popup.h"
#include "localizationUtils.h"
#include "HwndFunctions.h"
#include "pim/Utils.h"
#ifdef WIN32_PLATFORM_WFSP
#include "funresourcesp.h"
#endif
#ifdef WIN32_PLATFORM_PSPC
#include "funresourceppc.h"
#endif


// Popup dialog

IMPLEMENT_DYNAMIC(Popup, CDialog)

Popup::Popup(CWnd* pParent /*=NULL*/)
	: CDialog(Popup::IDD, pParent)
{
    
}

Popup::~Popup()
{
}

int Popup::DoModal()
{
    m_hInstSave = afxCurrentInstanceHandle;
    HINSTANCE h = AfxFindResourceHandle(MAKEINTRESOURCE(IDD), RT_DIALOG);
    afxCurrentInstanceHandle = h;
    INT_PTR m = CDialog::DoModal();    
    return m;
}

void Popup::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BOOL Popup::OnInitDialog(){
    
    afxCurrentInstanceHandle = m_hInstSave;   
    CString s1; s1.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_DOWNLOADING_TITLE);            
    SetWindowText(s1);
    CDialog::OnInitDialog();    
    return TRUE;
}

BEGIN_MESSAGE_MAP(Popup, CDialog)

    ON_WM_PAINT()
    ON_BN_CLICKED(IDCANCEL,             &Popup::OnBnClickedStop)
    ON_MESSAGE(ID_MYMSG_CLOSING_POPUP,  &Popup::OnBnTempStop)
    ON_MESSAGE(ID_MYMSG_SET_RANGE_PROGRESS,  &Popup::OnSetRange)    
    ON_MESSAGE(ID_MYMSG_GO_STEP_PROGRESS,  &Popup::OnGoStep)
    ON_WM_CLOSE()
   
END_MESSAGE_MAP()


void Popup::drawPopup(){
    
    CString s1; s1.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_DOWNLOAD_ATTACH_IN_PROGRESS);                
    SetDlgItemText(IDC_POPUP_STARTSYNC_TEXT, s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(),IDS_CANCEL);
    SetDlgItemText(IDCANCEL, s1);
    
    double yy = GetSystemMetrics(SM_CYSCREEN);
    double xx = GetSystemMetrics(SM_CXSCREEN);
    
    int moltiplicator = 1;
    int tmp = 0;
    if (isVGAScreen(&tmp, &tmp) || isQVGASquareScreen(&tmp, &tmp)) {
        moltiplicator = 2;
    }
    //Setting up the new size of the Dialog
    int yPerc  = 45; //45% of the client length (Y)
    int xPerc  = 80; //80% of the client width (X)
    int messageyPerc = yPerc - 35;
    int buttonyPerc  = yPerc + 5;
    
    if (yy == 240) { // for square screen
        yPerc = 60;        
    }    
    
    //Setting up the new size of the Dialog
    int y = (int)((yy/100)*yPerc); //45% of the client y
    int x = (int)((xx/100)*xPerc); //80% of the client x

    //setting up the position of the Dialog. It will be centered
    int dy = (int)((yy - y) /2);
    int dx = (int)((xx - x) /2);

    //Ok let's resize and move the Dialog
    SetWindowPos(GetActiveWindow(), dx, dy, x, y, SWP_SHOWWINDOW);
    
    int step1 = (int)(0.03 * y);
    int step2 = (int)(0.1 * y);
    // from top
    int y1_size = (int)(0.25 * y);
    int y2_size = (int)(0.15 * y) ;
    int y3_size = (int)(0.15 * y);

    int x_size_message = (int)(0.8 * x);
    int x_size_progress = (int)(0.8 * x);
    int x_size_button = (int)(0.4 * x);
        
    // The values are relative to the popup window
    int x_pos = (int)(((1 - 0.8) / 2) * x); // the same for message and progress bar
    int x_pos_butt = (int)(((1 - 0.4) / 2) * x); // only for the button
    
    int y_pos = (int)(0.05 * y);
    

     // Now we resize the text box
    GetDlgItem(IDC_POPUP_STARTSYNC_TEXT)->SetWindowPos(GetActiveWindow(), x_pos, y_pos,
        x_size_message, y1_size, SWP_NOZORDER);

    
    y_pos += y1_size;
    y_pos += step1;

    GetDlgItem(IDC_PROGRESS1)->SetWindowPos(GetActiveWindow(), x_pos, y_pos,
        x_size_progress, y2_size, SWP_NOZORDER);
    
    y_pos += y2_size;
    y_pos += step2;

    GetDlgItem(IDCANCEL)->SetWindowPos(GetActiveWindow(), x_pos_butt, y_pos,
        x_size_button, y3_size, SWP_NOZORDER);

}

void Popup::OnPaint() {

    CDialog::OnPaint();
    this->drawPopup();
}

void Popup::OnBnClickedStop()
{   
    this->EndDialog(1);
}

void Popup::OnClose() {    
    this->EndDialog(0);
}

LRESULT Popup::OnBnTempStop(WPARAM code, LPARAM lparam) {

    this->EndDialog(code);
    return code;
}

LRESULT Popup::OnSetRange(WPARAM wparam, LPARAM range) {
    HWND h = GetDlgItem(IDC_PROGRESS1)->GetSafeHwnd();
    ::SendMessage(h, PBM_SETRANGE, 0, MAKELPARAM(0, range)); 
    return ::SendMessage(h, PBM_SETSTEP, (WPARAM) 1, 0);

}

LRESULT Popup::OnGoStep(WPARAM code, LPARAM lparam) {
    HWND h = GetDlgItem(IDC_PROGRESS1)->GetSafeHwnd();
    return ::SendMessage(h, PBM_STEPIT, 0, 0); 

}