﻿/////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2007, Microsoft All Rights Reserved.
//
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
/////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.DeviceEmulatorManager.Interop;
using System.Runtime.InteropServices;

namespace WindowsMobileDeveloperSamples.DeviceEmulatorManagerAutomationWrapper
{
    /// <summary>
    /// Represents a Category within the Device Emulator Manager 
    /// </summary>
    public class DeviceEmulatorCategory
    {
        const int END_OF_DATA = -2147024637;

        internal DeviceEmulatorCategory(DeviceEmulatorManagerEx mgr)
        {
            _name = mgr.NativeDeviceEmulatorManager.get_Name();

            _nativeEnumManagerSDKs = mgr.NativeDeviceEmulatorManager.EnumerateSDKs();
        }

        string _name;
        /// <summary>
        /// Gets the name of the current node such as My Device Emulators, All Device Emulators, or Others.
        /// <remarks>
        /// See the IDeviceEmulatorManager::get_Name documentation for more details
        /// </remarks>
        /// </summary>
        public string Name { get { return _name; } }

        // Underlying Native COM Interface Reference
        private IEnumManagerSDKs _nativeEnumManagerSDKs;
        internal IEnumManagerSDKs NativeEnumManagerSDKs { get { return _nativeEnumManagerSDKs; } }

        public override string ToString()
        {
            return _name;
        }

        private DeviceEmulatorSdkCollection _sdks = null;
        /// <summary>
        /// The list of SDKs within this category.
        /// </summary>
        public DeviceEmulatorSdkCollection Sdks
        {
            get
            {
                if (_sdks == null)
                {
                    List<DeviceEmulatorSdk> sdkList = BuildSdkList();
                    _sdks = new DeviceEmulatorSdkCollection(sdkList);
                }

                return _sdks;
            }
        }

        private List<DeviceEmulatorSdk> BuildSdkList()
        {
            List<DeviceEmulatorSdk> sdkList = new List<DeviceEmulatorSdk>();
            DeviceEmulatorSdk sdk;

            try
            {
                _nativeEnumManagerSDKs.Reset();
                while (true)
                {
                    sdk = new DeviceEmulatorSdk(this);
                    sdkList.Add(sdk);
                    _nativeEnumManagerSDKs.MoveNext();
                }
            }
            catch (COMException ex)
            {
                if (ex.ErrorCode != END_OF_DATA &&
                    ex.ErrorCode != DeviceEmulatorManagerErrorCodes.E_ENUMSDK_NOT_LOADED)
                    throw ex;
            }

            return sdkList;
        }

    }
}
