﻿/////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2007, Microsoft All Rights Reserved.
//
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
/////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace WindowsMobileDeveloperSamples.DeviceEmulatorManagerAutomationWrapper
{
    public class DeviceEmulatorCategoryCollection : ReadOnlyCollection<DeviceEmulatorCategory>
    {
        internal DeviceEmulatorCategoryCollection(List<DeviceEmulatorCategory> categories)
            : base(categories)
        { }

        /// <summary>
        /// Returns the Category with the name matching categoryName
        /// </summary>
        /// <param name="sdkName">Name of the desired Category</param>
        /// <returns>DeviceEmulatorCategory instance with the specified name</returns>
        public DeviceEmulatorCategory this[string categoryName]
        {
            get
            {
                DeviceEmulatorCategory returnValue = null;
                foreach (DeviceEmulatorCategory category in this)
                {
                    if (string.Compare(category.Name, categoryName, true) == 0)
                    {
                        returnValue = category;
                        break;
                    }
                }
                // not found
                if (returnValue == null)
                    throw new IndexOutOfRangeException("No DeviceEmulatorCategory exists with the name:" + categoryName);

                return returnValue;
            }
        }
    }
}
