﻿/////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2007, Microsoft All Rights Reserved.
//
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
/////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace WindowsMobileDeveloperSamples.DeviceEmulatorManagerAutomationWrapper 
{
    public class DeviceEmulatorCollection : ReadOnlyCollection<DeviceEmulator>
    {
        internal DeviceEmulatorCollection(List<DeviceEmulator> emulators)
            : base(emulators)
        {}

        /// <summary>
        /// Returns the Device Emulator whose Name or VMID matches the value in emulatorNameOrVMID
        /// </summary>
        /// <param name="sdkName">Name or VMID of the desired Device Emulator</param>
        /// <returns>DeviceEmulator instance with the specified name or VMID</returns>
        public DeviceEmulator this[string emulatorNameOrVMID]
        {
            get
            {
                DeviceEmulator returnValue = null;
                foreach (DeviceEmulator deviceEmulator in this)
                {
                    if (string.Compare(deviceEmulator.Name, emulatorNameOrVMID, true) == 0 ||
                        string.Compare(deviceEmulator.VMID, emulatorNameOrVMID, true) == 0)
                    {
                        returnValue = deviceEmulator;
                        break;
                    }
                }
                // not found
                if (returnValue == null)
                    throw new IndexOutOfRangeException("No DeviceEmulator exists with the name or VMID:" + emulatorNameOrVMID);

                return returnValue;
            }
        }
    }
}
