﻿/////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2007, Microsoft All Rights Reserved.
//
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES OR INDEMNITIES.
//
/////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.DeviceEmulatorManager.Interop;
using System.Runtime.InteropServices;

namespace WindowsMobileDeveloperSamples.DeviceEmulatorManagerAutomationWrapper
{
    /// <summary>
    /// Represents an SDK (sometimes referred to as platform) with a Category
    /// </summary>
    public class DeviceEmulatorSdk
    {
        const int END_OF_DATA = -2147024637;

        internal DeviceEmulatorSdk(DeviceEmulatorCategory category)
        {
            _name = category.NativeEnumManagerSDKs.get_Name();
            _nativeEnumVMIDs = category.NativeEnumManagerSDKs.EnumerateVMIDs();
        }

        // Underlying Native COM Interface Reference
        private IEnumVMIDs _nativeEnumVMIDs;
        internal IEnumVMIDs NativeEnumVMIDs { get { return _nativeEnumVMIDs; } }

        string _name;
        /// <summary>
        /// Gets the name of the current platform such as Windows Mobile 5.0 Smartphone SDK.
        /// <remarks>
        /// See the IEnumManagerSDKs::get_Name documentation for more details
        /// </remarks>
        /// </summary>
        public string Name { get { return _name; } }

        public override string ToString()
        {
            return _name;
        }

        private DeviceEmulatorCollection _deviceEmulators = null;
        /// <summary>
        /// The list of emulators within this SDK
        /// </summary>
        public DeviceEmulatorCollection DeviceEmulators
        {
            get
            {
                if (_deviceEmulators == null)
                {
                    List<DeviceEmulator> deviceEmulatorList = BuildDeviceEmulatorList();
                    _deviceEmulators = new DeviceEmulatorCollection(deviceEmulatorList);
                }

                return _deviceEmulators;
            }
        }

        private List<DeviceEmulator> BuildDeviceEmulatorList()
        {
            List<DeviceEmulator> deviceEmulatorList = new List<DeviceEmulator>();
            DeviceEmulator deviceEmulator;

            _nativeEnumVMIDs.Reset();
            try
            {
                while (true)
                {
                    deviceEmulator = new DeviceEmulator(this);
                    deviceEmulatorList.Add(deviceEmulator);
                    _nativeEnumVMIDs.MoveNext();
                }
            }
            catch (COMException ex)
            {
                // End of data
                if (ex.ErrorCode != END_OF_DATA &&
                    ex.ErrorCode != DeviceEmulatorManagerErrorCodes.E_ENUMVMID_INVALID_VMID &&
                    ex.ErrorCode != DeviceEmulatorManagerErrorCodes.E_ENUMVMID_NOT_LOADED)
                    throw ex;
            }

            return deviceEmulatorList;
        }
    }

}
