﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Data;
using System.Linq;
using Microsoft.DeviceEmulatorManager.Interop;
using System.Runtime.InteropServices;
using WindowsMobileDeveloperSamples.DeviceEmulatorManagerAutomationWrapper;
using System.Threading;
using System.Diagnostics;

namespace FunambolDeviceEmulatorManager
{
    class Launcher
    {
        static string WM6_professional = "Windows Mobile 6 Professional Emulator";
        static string WM6_standard = "Windows Mobile 6 Standard QVGA Emulator";

        static void Main(string[] args)
        {
            if (args.Length == 0 || args.Length == 1)
            {
                Console.WriteLine("\n\tusage: DMHandler [start|stop|shutdown] [emulator number] ");
                Console.WriteLine("\t[emulator number] could be:");
                Console.WriteLine("\t\t 1 - Windows Mobile 6 Professional Emulator");
                Console.WriteLine("\t\t 2 - USA Windows Mobile 6.1 Standard QVGA Emulator");
                Console.WriteLine("\t\nExample: DMHandler start 1");

                return;
            }

            Launcher p = new Launcher();

            int val = Int32.Parse(args[1]);
            if ("start" == args[0])
            {
                p.start(val);
            }
            else if ("stop" == args[0])
            {
                p.stop(val);
            }
            else if ("shutdown" == args[0])
            {
                p.shutdown(val);
            }
            else
                return;
        }

        public DeviceEmulator getDeviceEmulator(DeviceEmulatorManagerEx dem, int emulator)
        {
            DeviceEmulator d = null;
            switch (emulator)
            {
                case 1:
                    d = dem.FindDeviceEmulator(WM6_professional);
                    break;
                case 2:
                    d = dem.FindDeviceEmulator(WM6_standard);
                    break;
                default:
                    break;
            }
            return d;
        }

        public void stop(int emulator)
        {
            DeviceEmulatorManagerEx demManager = new DeviceEmulatorManagerEx();
            DeviceEmulator deviceEmulator = getDeviceEmulator(demManager, emulator);
            deviceEmulator.UnCradle();
            demManager.Dispose(); // always call Dispose when done
        }

        public void shutdown(int emulator)
        {
            DeviceEmulatorManagerEx demManager = new DeviceEmulatorManagerEx();
            DeviceEmulator deviceEmulator = getDeviceEmulator(demManager, emulator);
            deviceEmulator.Shutdown(false);
            demManager.Dispose(); // always call Dispose when done
        }

        public bool startEmulatorManager()
        {

            ProcessStartInfo p = new ProcessStartInfo();
            p.FileName = "dvcemumanager.exe";
            Process proc = Process.Start(p);
            int ret = proc.Id;
            if (proc.Id != 0)
            {
                Thread.Sleep(5000);
                return true;
            }
            else
            {
                Console.WriteLine("\n\tEmulatorManager (dvcemumanager.exe) not found in PATH");
                Console.WriteLine("\tSet it and try again");
                return false;
            }

        }

        public void start(int emulator)
        {
            if (!startEmulatorManager())
            {
                return;
            }
            DeviceEmulatorManagerEx demManager = new DeviceEmulatorManagerEx();
            DeviceEmulatorCategory dataStore = demManager.Categories["DataStore"];
            DeviceEmulatorSdk wm6ProSdk = demManager.Categories["DataStore"].Sdks["Windows Mobile 6 Professional SDK"];
            DeviceEmulator wm6Pro = demManager.Categories["DataStore"].Sdks["Windows Mobile 6 Professional SDK"].DeviceEmulators["Windows Mobile 6 Professional Emulator"];

            DeviceEmulator deviceEmulator = getDeviceEmulator(demManager, emulator);
            if (deviceEmulator != null)
            {
                if (deviceEmulator.State == EMULATOR_STATE.EMU_NOT_RUNNING)
                {
                    deviceEmulator.Connect();
                    Thread.Sleep(10000);
                }
                else
                {
                    deviceEmulator.BringToFront(); // already running - bring to front            
                }
                deviceEmulator.Cradle();
            }
            demManager.Dispose();
        }
    }
}
